# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 Simon Ser
# Copyright © 2019 Ivan Molodetskikh
# Copyright © 2024 Neal Gompa
#
# Permission to use, copy, modify, distribute, and sell this
# software and its documentation for any purpose is hereby granted
# without fee, provided that the above copyright notice appear in
# all copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of
# the copyright holders not be used in advertising or publicity
# pertaining to distribution of the software without specific,
# written prior permission.  The copyright holders make no
# representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied
# warranty.
#
# THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
# SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
# THIS SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSeat
from .wayland import WlSeatProxy


class ExtDataControlDeviceV1(Interface):
    """Manage a data device for a seat

    This interface allows a client to manage a seat's selection.

    When the seat is destroyed, this object becomes inert.
    """

    name = "ext_data_control_device_v1"
    version = 1

    class error(enum.IntEnum):
        used_source = 1


class ExtDataControlManagerV1(Interface):
    """Manager to control data devices

    This interface is a manager that allows creating per-seat data device
    controls.
    """

    name = "ext_data_control_manager_v1"
    version = 1


class ExtDataControlOfferV1(Interface):
    """Offer to transfer data

    A ext_data_control_offer represents a piece of data offered for transfer by
    another client (the source client). The offer describes the different MIME
    types that the data can be converted to and provides the mechanism for
    transferring the data directly from the source client.
    """

    name = "ext_data_control_offer_v1"
    version = 1


class ExtDataControlSourceV1(Interface):
    """Offer to transfer data

    The ext_data_control_source object is the source side of a
    ext_data_control_offer. It is created by the source client in a data
    transfer and provides a way to describe the offered data and a way to
    respond to requests to transfer the data.
    """

    name = "ext_data_control_source_v1"
    version = 1

    class error(enum.IntEnum):
        invalid_offer = 1


class ExtDataControlDeviceV1Resource(Resource[ExtDataControlDeviceV1]):
    interface = ExtDataControlDeviceV1

    @ExtDataControlDeviceV1.event(
        Argument(ArgumentType.NewId, interface=ExtDataControlOfferV1),
    )
    def data_offer(self, id: ExtDataControlOfferV1) -> None:
        """Introduce a new ext_data_control_offer

        The data_offer event introduces a new ext_data_control_offer object,
        which will subsequently be used in either the
        ext_data_control_device.selection event (for the regular clipboard
        selections) or the ext_data_control_device.primary_selection event (for
        the primary clipboard selections). Immediately following the
        ext_data_control_device.data_offer event, the new data_offer object
        will send out ext_data_control_offer.offer events to describe the MIME
        types it offers.

        :param id:
        :type id:
            :class:`~pywayland.protocol.ext_data_control_v1.ExtDataControlOfferV1`
        """
        self._post_event(0, id)

    @ExtDataControlDeviceV1.event(
        Argument(ArgumentType.Object, interface=ExtDataControlOfferV1, nullable=True),
    )
    def selection(self, id: ExtDataControlOfferV1 | None) -> None:
        """Advertise new selection

        The selection event is sent out to notify the client of a new
        ext_data_control_offer for the selection for this device. The
        ext_data_control_device.data_offer and the ext_data_control_offer.offer
        events are sent out immediately before this event to introduce the data
        offer object. The selection event is sent to a client when a new
        selection is set. The ext_data_control_offer is valid until a new
        ext_data_control_offer or NULL is received. The client must destroy the
        previous selection ext_data_control_offer, if any, upon receiving this
        event. Regardless, the previous selection will be ignored once a new
        selection ext_data_control_offer is received.

        The first selection event is sent upon binding the
        ext_data_control_device object.

        :param id:
        :type id:
            :class:`~pywayland.protocol.ext_data_control_v1.ExtDataControlOfferV1` or `None`
        """
        self._post_event(1, id)

    @ExtDataControlDeviceV1.event()
    def finished(self) -> None:
        """This data control is no longer valid

        This data control object is no longer valid and should be destroyed by
        the client.
        """
        self._post_event(2)

    @ExtDataControlDeviceV1.event(
        Argument(ArgumentType.Object, interface=ExtDataControlOfferV1, nullable=True),
    )
    def primary_selection(self, id: ExtDataControlOfferV1 | None) -> None:
        """Advertise new primary selection

        The primary_selection event is sent out to notify the client of a new
        ext_data_control_offer for the primary selection for this device. The
        ext_data_control_device.data_offer and the ext_data_control_offer.offer
        events are sent out immediately before this event to introduce the data
        offer object. The primary_selection event is sent to a client when a
        new primary selection is set. The ext_data_control_offer is valid until
        a new ext_data_control_offer or NULL is received. The client must
        destroy the previous primary selection ext_data_control_offer, if any,
        upon receiving this event. Regardless, the previous primary selection
        will be ignored once a new primary selection ext_data_control_offer is
        received.

        If the compositor supports primary selection, the first
        primary_selection event is sent upon binding the
        ext_data_control_device object.

        :param id:
        :type id:
            :class:`~pywayland.protocol.ext_data_control_v1.ExtDataControlOfferV1` or `None`
        """
        self._post_event(3, id)


class ExtDataControlManagerV1Resource(Resource[ExtDataControlManagerV1]):
    interface = ExtDataControlManagerV1


class ExtDataControlOfferV1Resource(Resource[ExtDataControlOfferV1]):
    interface = ExtDataControlOfferV1

    @ExtDataControlOfferV1.event(
        Argument(ArgumentType.String),
    )
    def offer(self, mime_type: str) -> None:
        """Advertise offered mime type

        Sent immediately after creating the ext_data_control_offer object. One
        event per offered MIME type.

        :param mime_type:
            offered MIME type
        :type mime_type:
            `ArgumentType.String`
        """
        self._post_event(0, mime_type)


class ExtDataControlSourceV1Resource(Resource[ExtDataControlSourceV1]):
    interface = ExtDataControlSourceV1

    @ExtDataControlSourceV1.event(
        Argument(ArgumentType.String),
        Argument(ArgumentType.FileDescriptor),
    )
    def send(self, mime_type: str, fd: int) -> None:
        """Send the data

        Request for data from the client. Send the data as the specified MIME
        type over the passed file descriptor, then close it.

        :param mime_type:
            MIME type for the data
        :type mime_type:
            `ArgumentType.String`
        :param fd:
            file descriptor for the data
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._post_event(0, mime_type, fd)

    @ExtDataControlSourceV1.event()
    def cancelled(self) -> None:
        """Selection was cancelled

        This data source is no longer valid. The data source has been replaced
        by another data source.

        The client should clean up and destroy this data source.
        """
        self._post_event(1)


class ExtDataControlDeviceV1Proxy(Proxy[ExtDataControlDeviceV1]):
    interface = ExtDataControlDeviceV1

    @ExtDataControlDeviceV1.request(
        Argument(ArgumentType.Object, interface=ExtDataControlSourceV1, nullable=True),
    )
    def set_selection(self, source: ExtDataControlSourceV1Proxy | None) -> None:
        """Copy data to the selection

        This request asks the compositor to set the selection to the data from
        the source on behalf of the client.

        The given source may not be used in any further set_selection or
        set_primary_selection requests. Attempting to use a previously used
        source triggers the used_source protocol error.

        To unset the selection, set the source to NULL.

        :param source:
        :type source:
            :class:`~pywayland.protocol.ext_data_control_v1.ExtDataControlSourceV1` or `None`
        """
        self._marshal(0, source)

    @ExtDataControlDeviceV1.request()
    def destroy(self) -> None:
        """Destroy this data device

        Destroys the data device object.
        """
        self._marshal(1)
        self._destroy()

    @ExtDataControlDeviceV1.request(
        Argument(ArgumentType.Object, interface=ExtDataControlSourceV1, nullable=True),
    )
    def set_primary_selection(self, source: ExtDataControlSourceV1Proxy | None) -> None:
        """Copy data to the primary selection

        This request asks the compositor to set the primary selection to the
        data from the source on behalf of the client.

        The given source may not be used in any further set_selection or
        set_primary_selection requests. Attempting to use a previously used
        source triggers the used_source protocol error.

        To unset the primary selection, set the source to NULL.

        The compositor will ignore this request if it does not support primary
        selection.

        :param source:
        :type source:
            :class:`~pywayland.protocol.ext_data_control_v1.ExtDataControlSourceV1` or `None`
        """
        self._marshal(2, source)


class ExtDataControlManagerV1Proxy(Proxy[ExtDataControlManagerV1]):
    interface = ExtDataControlManagerV1

    @ExtDataControlManagerV1.request(
        Argument(ArgumentType.NewId, interface=ExtDataControlSourceV1),
    )
    def create_data_source(self) -> ExtDataControlSourceV1Proxy:
        """Create a new data source

        Create a new data source.

        :returns:
            :class:`~pywayland.protocol.ext_data_control_v1.ExtDataControlSourceV1`
            -- data source to create
        """
        id = self._marshal_constructor(0, ExtDataControlSourceV1)
        return id

    @ExtDataControlManagerV1.request(
        Argument(ArgumentType.NewId, interface=ExtDataControlDeviceV1),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def get_data_device(self, seat: WlSeatProxy) -> ExtDataControlDeviceV1Proxy:
        """Get a data device for a seat

        Create a data device that can be used to manage a seat's selection.

        :param seat:
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :returns:
            :class:`~pywayland.protocol.ext_data_control_v1.ExtDataControlDeviceV1`
        """
        id = self._marshal_constructor(1, ExtDataControlDeviceV1, seat)
        return id

    @ExtDataControlManagerV1.request()
    def destroy(self) -> None:
        """Destroy the manager

        All objects created by the manager will still remain valid, until their
        appropriate destroy request has been called.
        """
        self._marshal(2)
        self._destroy()


class ExtDataControlOfferV1Proxy(Proxy[ExtDataControlOfferV1]):
    interface = ExtDataControlOfferV1

    @ExtDataControlOfferV1.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.FileDescriptor),
    )
    def receive(self, mime_type: str, fd: int) -> None:
        """Request that the data is transferred

        To transfer the offered data, the client issues this request and
        indicates the MIME type it wants to receive. The transfer happens
        through the passed file descriptor (typically created with the pipe
        system call). The source client writes the data in the MIME type
        representation requested and then closes the file descriptor.

        The receiving client reads from the read end of the pipe until EOF and
        then closes its end, at which point the transfer is complete.

        This request may happen multiple times for different MIME types.

        :param mime_type:
            MIME type desired by receiver
        :type mime_type:
            `ArgumentType.String`
        :param fd:
            file descriptor for data transfer
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._marshal(0, mime_type, fd)

    @ExtDataControlOfferV1.request()
    def destroy(self) -> None:
        """Destroy this offer

        Destroys the data offer object.
        """
        self._marshal(1)
        self._destroy()


class ExtDataControlSourceV1Proxy(Proxy[ExtDataControlSourceV1]):
    interface = ExtDataControlSourceV1

    @ExtDataControlSourceV1.request(
        Argument(ArgumentType.String),
    )
    def offer(self, mime_type: str) -> None:
        """Add an offered mime type

        This request adds a MIME type to the set of MIME types advertised to
        targets. Can be called several times to offer multiple types.

        Calling this after ext_data_control_device.set_selection is a protocol
        error.

        :param mime_type:
            MIME type offered by the data source
        :type mime_type:
            `ArgumentType.String`
        """
        self._marshal(0, mime_type)

    @ExtDataControlSourceV1.request()
    def destroy(self) -> None:
        """Destroy this source

        Destroys the data source object.
        """
        self._marshal(1)
        self._destroy()


class ExtDataControlDeviceV1Global(Global[ExtDataControlDeviceV1]):
    interface = ExtDataControlDeviceV1


class ExtDataControlManagerV1Global(Global[ExtDataControlManagerV1]):
    interface = ExtDataControlManagerV1


class ExtDataControlOfferV1Global(Global[ExtDataControlOfferV1]):
    interface = ExtDataControlOfferV1


class ExtDataControlSourceV1Global(Global[ExtDataControlSourceV1]):
    interface = ExtDataControlSourceV1


ExtDataControlDeviceV1._gen_c()
ExtDataControlDeviceV1.proxy_class = ExtDataControlDeviceV1Proxy
ExtDataControlDeviceV1.resource_class = ExtDataControlDeviceV1Resource
ExtDataControlDeviceV1.global_class = ExtDataControlDeviceV1Global


ExtDataControlManagerV1._gen_c()
ExtDataControlManagerV1.proxy_class = ExtDataControlManagerV1Proxy
ExtDataControlManagerV1.resource_class = ExtDataControlManagerV1Resource
ExtDataControlManagerV1.global_class = ExtDataControlManagerV1Global


ExtDataControlOfferV1._gen_c()
ExtDataControlOfferV1.proxy_class = ExtDataControlOfferV1Proxy
ExtDataControlOfferV1.resource_class = ExtDataControlOfferV1Resource
ExtDataControlOfferV1.global_class = ExtDataControlOfferV1Global


ExtDataControlSourceV1._gen_c()
ExtDataControlSourceV1.proxy_class = ExtDataControlSourceV1Proxy
ExtDataControlSourceV1.resource_class = ExtDataControlSourceV1Resource
ExtDataControlSourceV1.global_class = ExtDataControlSourceV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 NXP
# Copyright © 2019 Status Research & Development GmbH.
# Copyright © 2021 Xaver Hugl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WpDrmLeaseConnectorV1(Interface):
    """A leasable drm connector

    Represents a DRM connector which is available for lease. These objects are
    created via :func:`WpDrmLeaseDeviceV1.connector()
    <pywayland.protocol.drm_lease_v1.WpDrmLeaseDeviceV1.connector>` events, and
    should be passed to lease requests via
    :func:`WpDrmLeaseRequestV1.request_connector()
    <pywayland.protocol.drm_lease_v1.WpDrmLeaseRequestV1.request_connector>`.
    Immediately after the :class:`WpDrmLeaseConnectorV1` object is created the
    compositor will send a name, a description, a connector_id and a done
    event. When the description is updated the compositor will send a
    description event followed by a done event.
    """

    name = "wp_drm_lease_connector_v1"
    version = 1


class WpDrmLeaseDeviceV1(Interface):
    """Lease device

    This protocol is used by Wayland compositors which act as Direct Rendering
    Manager (DRM) masters to lease DRM resources to Wayland clients.

    The compositor will advertise one :class:`WpDrmLeaseDeviceV1` global for
    each DRM node. Some time after a client binds to the
    :class:`WpDrmLeaseDeviceV1` global, the compositor will send a drm_fd event
    followed by zero, one or more connector events. After all currently
    available connectors have been sent, the compositor will send a
    :func:`WpDrmLeaseDeviceV1.done()` event.

    When the list of connectors available for lease changes the compositor will
    send :func:`WpDrmLeaseDeviceV1.connector()` events for added connectors and
    :func:`WpDrmLeaseConnectorV1.withdrawn()
    <pywayland.protocol.drm_lease_v1.WpDrmLeaseConnectorV1.withdrawn>` events
    for removed connectors, followed by a :func:`WpDrmLeaseDeviceV1.done()`
    event.

    The compositor will indicate when a device is gone by removing the global
    via a :func:`WlRegistry.global_remove()
    <pywayland.protocol.wayland.WlRegistry.global_remove>` event. Upon
    receiving this event, the client should destroy any matching
    :class:`WpDrmLeaseDeviceV1` object.

    To destroy a :class:`WpDrmLeaseDeviceV1` object, the client must first
    issue a release request. Upon receiving this request, the compositor will
    immediately send a released event and destroy the object. The client must
    continue to process and discard drm_fd and connector events until it
    receives the released event. Upon receiving the released event, the client
    can safely cleanup any client-side resources.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_drm_lease_device_v1"
    version = 1


class WpDrmLeaseRequestV1(Interface):
    """Drm lease request

    A client that wishes to lease DRM resources will attach the list of
    connectors advertised with :func:`WpDrmLeaseDeviceV1.connector()
    <pywayland.protocol.drm_lease_v1.WpDrmLeaseDeviceV1.connector>` that they
    wish to lease, then use :func:`WpDrmLeaseRequestV1.submit()` to submit the
    request.
    """

    name = "wp_drm_lease_request_v1"
    version = 1

    class error(enum.IntEnum):
        wrong_device = 0
        duplicate_connector = 1
        empty_lease = 2


class WpDrmLeaseV1(Interface):
    """A drm lease

    A DRM lease object is used to transfer the DRM file descriptor to the
    client and manage the lifetime of the lease.

    Some time after the :class:`WpDrmLeaseV1` object is created, the compositor
    will reply with the lease request's result. If the lease request is
    granted, the compositor will send a lease_fd event. If the lease request is
    denied, the compositor will send a finished event without a lease_fd event.
    """

    name = "wp_drm_lease_v1"
    version = 1


class WpDrmLeaseConnectorV1Resource(Resource[WpDrmLeaseConnectorV1]):
    interface = WpDrmLeaseConnectorV1

    @WpDrmLeaseConnectorV1.event(
        Argument(ArgumentType.String),
    )
    def name(self, name: str) -> None:
        """Name

        The compositor sends this event once the connector is created to
        indicate the name of this connector. This will not change for the
        duration of the Wayland session, but is not guaranteed to be consistent
        between sessions.

        If the compositor supports
        :class:`~pywayland.protocol.wayland.WlOutput` version 4 and this
        connector corresponds to a
        :class:`~pywayland.protocol.wayland.WlOutput`, the compositor should
        use the same name as for the
        :class:`~pywayland.protocol.wayland.WlOutput`.

        :param name:
            connector name
        :type name:
            `ArgumentType.String`
        """
        self._post_event(0, name)

    @WpDrmLeaseConnectorV1.event(
        Argument(ArgumentType.String),
    )
    def description(self, description: str) -> None:
        """Description

        The compositor sends this event once the connector is created to
        provide a human-readable description for this connector, which may be
        presented to the user. The compositor may send this event multiple
        times over the lifetime of this object to reflect changes in the
        description.

        :param description:
            connector description
        :type description:
            `ArgumentType.String`
        """
        self._post_event(1, description)

    @WpDrmLeaseConnectorV1.event(
        Argument(ArgumentType.Uint),
    )
    def connector_id(self, connector_id: int) -> None:
        """Connector_id

        The compositor sends this event once the connector is created to
        indicate the DRM object ID which represents the underlying connector
        that is being offered. Note that the final lease may include additional
        object IDs, such as CRTCs and planes.

        :param connector_id:
            DRM connector ID
        :type connector_id:
            `ArgumentType.Uint`
        """
        self._post_event(2, connector_id)

    @WpDrmLeaseConnectorV1.event()
    def done(self) -> None:
        """All properties have been sent

        This event is sent after all properties of a connector have been sent.
        This allows changes to the properties to be seen as atomic even if they
        happen via multiple events.
        """
        self._post_event(3)

    @WpDrmLeaseConnectorV1.event()
    def withdrawn(self) -> None:
        """Lease offer withdrawn

        Sent to indicate that the compositor will no longer honor requests for
        DRM leases which include this connector. The client may still issue a
        lease request including this connector, but the compositor will send
        :func:`WpDrmLeaseV1.finished()
        <pywayland.protocol.drm_lease_v1.WpDrmLeaseV1.finished>` without
        issuing a lease fd. Compositors are encouraged to send this event when
        they lose access to connector, for example when the connector is hot-
        unplugged, when the connector gets leased to a client or when the
        compositor loses DRM master.

        If a client holds a lease for the connector, the status of the lease
        remains the same. The client should destroy the object after receiving
        this event.
        """
        self._post_event(4)


class WpDrmLeaseDeviceV1Resource(Resource[WpDrmLeaseDeviceV1]):
    interface = WpDrmLeaseDeviceV1

    @WpDrmLeaseDeviceV1.event(
        Argument(ArgumentType.FileDescriptor),
    )
    def drm_fd(self, fd: int) -> None:
        """Open a non-master fd for this drm node

        The compositor will send this event when the
        :class:`WpDrmLeaseDeviceV1` global is bound, although there are no
        guarantees as to how long this takes - the compositor might need to
        wait until regaining DRM master. The included fd is a non-master DRM
        file descriptor opened for this device and the compositor must not
        authenticate it. The purpose of this event is to give the client the
        ability to query DRM and discover information which may help them pick
        the appropriate DRM device or select the appropriate connectors
        therein.

        :param fd:
            DRM file descriptor
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._post_event(0, fd)

    @WpDrmLeaseDeviceV1.event(
        Argument(ArgumentType.NewId, interface=WpDrmLeaseConnectorV1),
    )
    def connector(self, id: WpDrmLeaseConnectorV1) -> None:
        """Advertise connectors available for leases

        The compositor will use this event to advertise connectors available
        for lease by clients. This object may be passed into a lease request to
        indicate the client would like to lease that connector, see
        :func:`WpDrmLeaseRequestV1.request_connector()
        <pywayland.protocol.drm_lease_v1.WpDrmLeaseRequestV1.request_connector>`
        for details. While the compositor will make a best effort to not send
        disconnected connectors, no guarantees can be made.

        The compositor must send the drm_fd event before sending connectors.
        After the drm_fd event it will send all available connectors but may
        send additional connectors at any time.

        :param id:
        :type id:
            :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseConnectorV1`
        """
        self._post_event(1, id)

    @WpDrmLeaseDeviceV1.event()
    def done(self) -> None:
        """Signals grouping of connectors

        The compositor will send this event to indicate that it has sent all
        currently available connectors after the client binds to the global or
        when it updates the connector list, for example on hotplug, drm master
        change or when a leased connector becomes available again. It will
        similarly send this event to group
        :func:`WpDrmLeaseConnectorV1.withdrawn()
        <pywayland.protocol.drm_lease_v1.WpDrmLeaseConnectorV1.withdrawn>`
        events of connectors of this device.
        """
        self._post_event(2)

    @WpDrmLeaseDeviceV1.event()
    def released(self) -> None:
        """The compositor has finished using the device

        This event is sent in response to the release request and indicates
        that the compositor is done sending connector events. The compositor
        will destroy this object immediately after sending the event and it
        will become invalid. The client should release any resources associated
        with this device after receiving this event.
        """
        self._post_event(3)


class WpDrmLeaseRequestV1Resource(Resource[WpDrmLeaseRequestV1]):
    interface = WpDrmLeaseRequestV1


class WpDrmLeaseV1Resource(Resource[WpDrmLeaseV1]):
    interface = WpDrmLeaseV1

    @WpDrmLeaseV1.event(
        Argument(ArgumentType.FileDescriptor),
    )
    def lease_fd(self, leased_fd: int) -> None:
        """Shares the drm file descriptor

        This event returns a file descriptor suitable for use with DRM-related
        ioctls. The client should use drmModeGetLease to enumerate the DRM
        objects which have been leased to them. The compositor guarantees it
        will not use the leased DRM objects itself until it sends the finished
        event. If the compositor cannot or will not grant a lease for the
        requested connectors, it will not send this event, instead sending the
        finished event.

        The compositor will send this event at most once during this objects
        lifetime.

        :param leased_fd:
            leased DRM file descriptor
        :type leased_fd:
            `ArgumentType.FileDescriptor`
        """
        self._post_event(0, leased_fd)

    @WpDrmLeaseV1.event()
    def finished(self) -> None:
        """Sent when the lease has been revoked

        The compositor uses this event to either reject a lease request, or if
        it previously sent a lease_fd, to notify the client that the lease has
        been revoked. If the client requires a new lease, they should destroy
        this object and submit a new lease request. The compositor will send no
        further events for this object after sending the finish event.
        Compositors should revoke the lease when any of the leased resources
        become unavailable, namely when a hot-unplug occurs or when the
        compositor loses DRM master. Compositors may advertise the connector
        for leasing again, if the resource is available, by sending the
        connector event through the
        :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseDeviceV1` interface.
        """
        self._post_event(1)


class WpDrmLeaseConnectorV1Proxy(Proxy[WpDrmLeaseConnectorV1]):
    interface = WpDrmLeaseConnectorV1

    @WpDrmLeaseConnectorV1.request()
    def destroy(self) -> None:
        """Destroy connector

        The client may send this request to indicate that it will not use this
        connector. Clients are encouraged to send this after receiving the
        "withdrawn" event so that the server can release the resources
        associated with this connector offer. Neither existing lease requests
        nor leases will be affected.
        """
        self._marshal(0)
        self._destroy()


class WpDrmLeaseDeviceV1Proxy(Proxy[WpDrmLeaseDeviceV1]):
    interface = WpDrmLeaseDeviceV1

    @WpDrmLeaseDeviceV1.request(
        Argument(ArgumentType.NewId, interface=WpDrmLeaseRequestV1),
    )
    def create_lease_request(self) -> WpDrmLeaseRequestV1Proxy:
        """Create a lease request object

        Creates a lease request object.

        See the documentation for
        :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseRequestV1` for
        details.

        :returns:
            :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseRequestV1`
        """
        id = self._marshal_constructor(0, WpDrmLeaseRequestV1)
        return id

    @WpDrmLeaseDeviceV1.request()
    def release(self) -> None:
        """Release this object

        Indicates the client no longer wishes to use this object. In response
        the compositor will immediately send the released event and destroy
        this object. It can however not guarantee that the client won't receive
        connector events before the released event. The client must not send
        any requests after this one, doing so will raise a
        :class:`~pywayland.protocol.wayland.WlDisplay` error. Existing
        connectors, lease request and leases will not be affected.
        """
        self._marshal(1)


class WpDrmLeaseRequestV1Proxy(Proxy[WpDrmLeaseRequestV1]):
    interface = WpDrmLeaseRequestV1

    @WpDrmLeaseRequestV1.request(
        Argument(ArgumentType.Object, interface=WpDrmLeaseConnectorV1),
    )
    def request_connector(self, connector: WpDrmLeaseConnectorV1Proxy) -> None:
        """Request a connector for this lease

        Indicates that the client would like to lease the given connector. This
        is only used as a suggestion, the compositor may choose to include any
        resources in the lease it issues, or change the set of leased resources
        at any time. Compositors are however encouraged to include the
        requested connector and other resources necessary to drive the
        connected output in the lease.

        Requesting a connector that was created from a different lease device
        than this lease request raises the wrong_device error. Requesting a
        connector twice will raise the duplicate_connector error.

        :param connector:
        :type connector:
            :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseConnectorV1`
        """
        self._marshal(0, connector)

    @WpDrmLeaseRequestV1.request(
        Argument(ArgumentType.NewId, interface=WpDrmLeaseV1),
    )
    def submit(self) -> WpDrmLeaseV1Proxy:
        """Submit the lease request

        Submits the lease request and creates a new
        :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseV1` object. After
        calling submit the compositor will immediately destroy this object,
        issuing any more requests will cause a
        :class:`~pywayland.protocol.wayland.WlDisplay` error. The compositor
        doesn't make any guarantees about the events of the lease object,
        clients cannot expect an immediate response. Not requesting any
        connectors before submitting the lease request will raise the
        empty_lease error.

        :returns:
            :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseV1`
        """
        id = self._marshal_constructor(1, WpDrmLeaseV1)
        return id


class WpDrmLeaseV1Proxy(Proxy[WpDrmLeaseV1]):
    interface = WpDrmLeaseV1

    @WpDrmLeaseV1.request()
    def destroy(self) -> None:
        """Destroys the lease object

        The client should send this to indicate that it no longer wishes to use
        this lease. The compositor should use drmModeRevokeLease on the
        appropriate file descriptor, if necessary.

        Upon destruction, the compositor should advertise the connector for
        leasing again by sending the connector event through the
        :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseDeviceV1` interface.
        """
        self._marshal(0)
        self._destroy()


class WpDrmLeaseConnectorV1Global(Global[WpDrmLeaseConnectorV1]):
    interface = WpDrmLeaseConnectorV1


class WpDrmLeaseDeviceV1Global(Global[WpDrmLeaseDeviceV1]):
    interface = WpDrmLeaseDeviceV1


class WpDrmLeaseRequestV1Global(Global[WpDrmLeaseRequestV1]):
    interface = WpDrmLeaseRequestV1


class WpDrmLeaseV1Global(Global[WpDrmLeaseV1]):
    interface = WpDrmLeaseV1


WpDrmLeaseConnectorV1._gen_c()
WpDrmLeaseConnectorV1.proxy_class = WpDrmLeaseConnectorV1Proxy
WpDrmLeaseConnectorV1.resource_class = WpDrmLeaseConnectorV1Resource
WpDrmLeaseConnectorV1.global_class = WpDrmLeaseConnectorV1Global


WpDrmLeaseDeviceV1._gen_c()
WpDrmLeaseDeviceV1.proxy_class = WpDrmLeaseDeviceV1Proxy
WpDrmLeaseDeviceV1.resource_class = WpDrmLeaseDeviceV1Resource
WpDrmLeaseDeviceV1.global_class = WpDrmLeaseDeviceV1Global


WpDrmLeaseRequestV1._gen_c()
WpDrmLeaseRequestV1.proxy_class = WpDrmLeaseRequestV1Proxy
WpDrmLeaseRequestV1.resource_class = WpDrmLeaseRequestV1Resource
WpDrmLeaseRequestV1.global_class = WpDrmLeaseRequestV1Global


WpDrmLeaseV1._gen_c()
WpDrmLeaseV1.proxy_class = WpDrmLeaseV1Proxy
WpDrmLeaseV1.resource_class = WpDrmLeaseV1Resource
WpDrmLeaseV1.global_class = WpDrmLeaseV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2016 The Chromium Authors.
# Copyright 2017 Intel Corporation
# Copyright 2018 Collabora, Ltd
# Copyright 2021 Simon Ser
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class WpLinuxDrmSyncobjManagerV1(Interface):
    """Global for providing explicit synchronization

    This global is a factory interface, allowing clients to request explicit
    synchronization for buffers on a per-surface basis.

    See
    :class:`~pywayland.protocol.linux_drm_syncobj_v1.WpLinuxDrmSyncobjSurfaceV1`
    for more information.
    """

    name = "wp_linux_drm_syncobj_manager_v1"
    version = 1

    class error(enum.IntEnum):
        surface_exists = 0
        invalid_timeline = 1


class WpLinuxDrmSyncobjSurfaceV1(Interface):
    """Per-surface explicit synchronization

    This object is an add-on interface for
    :class:`~pywayland.protocol.wayland.WlSurface` to enable explicit
    synchronization.

    Each surface can be associated with only one object of this interface at
    any time.

    Explicit synchronization is guaranteed to be supported for buffers created
    with any version of the linux-dmabuf protocol. Compositors are free to
    support explicit synchronization for additional buffer types. If at surface
    commit time the attached buffer does not support explicit synchronization,
    an unsupported_buffer error is raised.

    As long as the :class:`WpLinuxDrmSyncobjSurfaceV1` object is alive, the
    compositor may ignore implicit synchronization for buffers attached and
    committed to the :class:`~pywayland.protocol.wayland.WlSurface`. The
    delivery of :func:`WlBuffer.release()
    <pywayland.protocol.wayland.WlBuffer.release>` events for buffers attached
    to the surface becomes undefined.

    Clients must set both acquire and release points if and only if a non-null
    buffer is attached in the same surface commit. See the no_buffer,
    no_acquire_point and no_release_point protocol errors.

    If at surface commit time the acquire and release DRM syncobj timelines are
    identical, the acquire point value must be strictly less than the release
    point value, or else the conflicting_points protocol error is raised.
    """

    name = "wp_linux_drm_syncobj_surface_v1"
    version = 1

    class error(enum.IntEnum):
        no_surface = 1
        unsupported_buffer = 2
        no_buffer = 3
        no_acquire_point = 4
        no_release_point = 5
        conflicting_points = 6


class WpLinuxDrmSyncobjTimelineV1(Interface):
    """Synchronization object timeline

    This object represents an explicit synchronization object timeline imported
    by the client to the compositor.
    """

    name = "wp_linux_drm_syncobj_timeline_v1"
    version = 1


class WpLinuxDrmSyncobjManagerV1Resource(Resource[WpLinuxDrmSyncobjManagerV1]):
    interface = WpLinuxDrmSyncobjManagerV1


class WpLinuxDrmSyncobjSurfaceV1Resource(Resource[WpLinuxDrmSyncobjSurfaceV1]):
    interface = WpLinuxDrmSyncobjSurfaceV1


class WpLinuxDrmSyncobjTimelineV1Resource(Resource[WpLinuxDrmSyncobjTimelineV1]):
    interface = WpLinuxDrmSyncobjTimelineV1


class WpLinuxDrmSyncobjManagerV1Proxy(Proxy[WpLinuxDrmSyncobjManagerV1]):
    interface = WpLinuxDrmSyncobjManagerV1

    @WpLinuxDrmSyncobjManagerV1.request()
    def destroy(self) -> None:
        """Destroy explicit synchronization factory object

        Destroy this explicit synchronization factory object. Other objects
        shall not be affected by this request.
        """
        self._marshal(0)
        self._destroy()

    @WpLinuxDrmSyncobjManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpLinuxDrmSyncobjSurfaceV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_surface(self, surface: WlSurfaceProxy) -> WpLinuxDrmSyncobjSurfaceV1Proxy:
        """Extend surface interface for explicit synchronization

        Instantiate an interface extension for the given
        :class:`~pywayland.protocol.wayland.WlSurface` to provide explicit
        synchronization.

        If the given :class:`~pywayland.protocol.wayland.WlSurface` already has
        an explicit synchronization object associated, the surface_exists
        protocol error is raised.

        Graphics APIs, like EGL or Vulkan, that manage the buffer queue and
        commits of a :class:`~pywayland.protocol.wayland.WlSurface` themselves,
        are likely to be using this extension internally. If a client is using
        such an API for a :class:`~pywayland.protocol.wayland.WlSurface`, it
        should not directly use this extension on that surface, to avoid
        raising a surface_exists protocol error.

        :param surface:
            the surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.linux_drm_syncobj_v1.WpLinuxDrmSyncobjSurfaceV1`
            -- the new synchronization surface object id
        """
        id = self._marshal_constructor(1, WpLinuxDrmSyncobjSurfaceV1, surface)
        return id

    @WpLinuxDrmSyncobjManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpLinuxDrmSyncobjTimelineV1),
        Argument(ArgumentType.FileDescriptor),
    )
    def import_timeline(self, fd: int) -> WpLinuxDrmSyncobjTimelineV1Proxy:
        """Import a drm syncobj timeline

        Import a DRM synchronization object timeline.

        If the FD cannot be imported, the invalid_timeline error is raised.

        :param fd:
            drm_syncobj file descriptor
        :type fd:
            `ArgumentType.FileDescriptor`
        :returns:
            :class:`~pywayland.protocol.linux_drm_syncobj_v1.WpLinuxDrmSyncobjTimelineV1`
        """
        id = self._marshal_constructor(2, WpLinuxDrmSyncobjTimelineV1, fd)
        return id


class WpLinuxDrmSyncobjSurfaceV1Proxy(Proxy[WpLinuxDrmSyncobjSurfaceV1]):
    interface = WpLinuxDrmSyncobjSurfaceV1

    @WpLinuxDrmSyncobjSurfaceV1.request()
    def destroy(self) -> None:
        """Destroy the surface synchronization object

        Destroy this surface synchronization object.

        Any timeline point set by this object with set_acquire_point or
        set_release_point since the last commit may be discarded by the
        compositor. Any timeline point set by this object before the last
        commit will not be affected.
        """
        self._marshal(0)
        self._destroy()

    @WpLinuxDrmSyncobjSurfaceV1.request(
        Argument(ArgumentType.Object, interface=WpLinuxDrmSyncobjTimelineV1),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def set_acquire_point(self, timeline: WpLinuxDrmSyncobjTimelineV1Proxy, point_hi: int, point_lo: int) -> None:
        """Set the acquire timeline point

        Set the timeline point that must be signalled before the compositor may
        sample from the buffer attached with :func:`WlSurface.attach()
        <pywayland.protocol.wayland.WlSurface.attach>`.

        The 64-bit unsigned value combined from point_hi and point_lo is the
        point value.

        The acquire point is double-buffered state, and will be applied on the
        next :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` request for the
        associated surface. Thus, it applies only to the buffer that is
        attached to the surface at commit time.

        If an acquire point has already been attached during the same commit
        cycle, the new point replaces the old one.

        If the associated :class:`~pywayland.protocol.wayland.WlSurface` was
        destroyed, a no_surface error is raised.

        If at surface commit time there is a pending acquire timeline point set
        but no pending buffer attached, a no_buffer error is raised. If at
        surface commit time there is a pending buffer attached but no pending
        acquire timeline point set, the no_acquire_point protocol error is
        raised.

        :param timeline:
        :type timeline:
            :class:`~pywayland.protocol.linux_drm_syncobj_v1.WpLinuxDrmSyncobjTimelineV1`
        :param point_hi:
            high 32 bits of the point value
        :type point_hi:
            `ArgumentType.Uint`
        :param point_lo:
            low 32 bits of the point value
        :type point_lo:
            `ArgumentType.Uint`
        """
        self._marshal(1, timeline, point_hi, point_lo)

    @WpLinuxDrmSyncobjSurfaceV1.request(
        Argument(ArgumentType.Object, interface=WpLinuxDrmSyncobjTimelineV1),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def set_release_point(self, timeline: WpLinuxDrmSyncobjTimelineV1Proxy, point_hi: int, point_lo: int) -> None:
        """Set the release timeline point

        Set the timeline point that must be signalled by the compositor when it
        has finished its usage of the buffer attached with
        :func:`WlSurface.attach()
        <pywayland.protocol.wayland.WlSurface.attach>` for the relevant commit.

        Once the timeline point is signaled, and assuming the associated buffer
        is not pending release from other :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` requests, no additional
        explicit or implicit synchronization with the compositor is required to
        safely re-use the buffer.

        Note that clients cannot rely on the release point being always
        signaled after the acquire point: compositors may release buffers
        without ever reading from them. In addition, the compositor may use
        different presentation paths for different commits, which may have
        different release behavior. As a result, the compositor may signal the
        release points in a different order than the client committed them.

        Because signaling a timeline point also signals every previous point,
        it is generally not safe to use the same timeline object for the
        release points of multiple buffers. The out-of-order signaling
        described above may lead to a release point being signaled before the
        compositor has finished reading. To avoid this, it is strongly
        recommended that each buffer should use a separate timeline for its
        release points.

        The 64-bit unsigned value combined from point_hi and point_lo is the
        point value.

        The release point is double-buffered state, and will be applied on the
        next :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` request for the
        associated surface. Thus, it applies only to the buffer that is
        attached to the surface at commit time.

        If a release point has already been attached during the same commit
        cycle, the new point replaces the old one.

        If the associated :class:`~pywayland.protocol.wayland.WlSurface` was
        destroyed, a no_surface error is raised.

        If at surface commit time there is a pending release timeline point set
        but no pending buffer attached, a no_buffer error is raised. If at
        surface commit time there is a pending buffer attached but no pending
        release timeline point set, the no_release_point protocol error is
        raised.

        :param timeline:
        :type timeline:
            :class:`~pywayland.protocol.linux_drm_syncobj_v1.WpLinuxDrmSyncobjTimelineV1`
        :param point_hi:
            high 32 bits of the point value
        :type point_hi:
            `ArgumentType.Uint`
        :param point_lo:
            low 32 bits of the point value
        :type point_lo:
            `ArgumentType.Uint`
        """
        self._marshal(2, timeline, point_hi, point_lo)


class WpLinuxDrmSyncobjTimelineV1Proxy(Proxy[WpLinuxDrmSyncobjTimelineV1]):
    interface = WpLinuxDrmSyncobjTimelineV1

    @WpLinuxDrmSyncobjTimelineV1.request()
    def destroy(self) -> None:
        """Destroy the timeline

        Destroy the synchronization object timeline. Other objects are not
        affected by this request, in particular timeline points set by
        set_acquire_point and set_release_point are not unset.
        """
        self._marshal(0)
        self._destroy()


class WpLinuxDrmSyncobjManagerV1Global(Global[WpLinuxDrmSyncobjManagerV1]):
    interface = WpLinuxDrmSyncobjManagerV1


class WpLinuxDrmSyncobjSurfaceV1Global(Global[WpLinuxDrmSyncobjSurfaceV1]):
    interface = WpLinuxDrmSyncobjSurfaceV1


class WpLinuxDrmSyncobjTimelineV1Global(Global[WpLinuxDrmSyncobjTimelineV1]):
    interface = WpLinuxDrmSyncobjTimelineV1


WpLinuxDrmSyncobjManagerV1._gen_c()
WpLinuxDrmSyncobjManagerV1.proxy_class = WpLinuxDrmSyncobjManagerV1Proxy
WpLinuxDrmSyncobjManagerV1.resource_class = WpLinuxDrmSyncobjManagerV1Resource
WpLinuxDrmSyncobjManagerV1.global_class = WpLinuxDrmSyncobjManagerV1Global


WpLinuxDrmSyncobjSurfaceV1._gen_c()
WpLinuxDrmSyncobjSurfaceV1.proxy_class = WpLinuxDrmSyncobjSurfaceV1Proxy
WpLinuxDrmSyncobjSurfaceV1.resource_class = WpLinuxDrmSyncobjSurfaceV1Resource
WpLinuxDrmSyncobjSurfaceV1.global_class = WpLinuxDrmSyncobjSurfaceV1Global


WpLinuxDrmSyncobjTimelineV1._gen_c()
WpLinuxDrmSyncobjTimelineV1.proxy_class = WpLinuxDrmSyncobjTimelineV1Proxy
WpLinuxDrmSyncobjTimelineV1.resource_class = WpLinuxDrmSyncobjTimelineV1Resource
WpLinuxDrmSyncobjTimelineV1.global_class = WpLinuxDrmSyncobjTimelineV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2020 - 2023 Andri Yngvason
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ExtTransientSeatManagerV1(Interface):
    """Transient seat manager

    The transient seat manager creates short-lived seats.
    """

    name = "ext_transient_seat_manager_v1"
    version = 1


class ExtTransientSeatV1(Interface):
    """Transient seat handle

    When the transient seat handle is destroyed, the seat itself will also be
    destroyed.
    """

    name = "ext_transient_seat_v1"
    version = 1


class ExtTransientSeatManagerV1Resource(Resource[ExtTransientSeatManagerV1]):
    interface = ExtTransientSeatManagerV1


class ExtTransientSeatV1Resource(Resource[ExtTransientSeatV1]):
    interface = ExtTransientSeatV1

    @ExtTransientSeatV1.event(
        Argument(ArgumentType.Uint),
    )
    def ready(self, global_name: int) -> None:
        """Transient seat is ready

        This event advertises the global name for the
        :class:`~pywayland.protocol.wayland.WlSeat` to be used with
        wl_registry_bind.

        It is sent exactly once, immediately after the transient seat is
        created and the new ":class:`~pywayland.protocol.wayland.WlSeat`"
        global is advertised, if and only if the creation of the transient seat
        was allowed.

        :param global_name:
        :type global_name:
            `ArgumentType.Uint`
        """
        self._post_event(0, global_name)

    @ExtTransientSeatV1.event()
    def denied(self) -> None:
        """Transient seat creation denied

        The event informs the client that the compositor denied its request to
        create a transient seat.

        It is sent exactly once, immediately after the transient seat object is
        created, if and only if the creation of the transient seat was denied.

        After receiving this event, the client should destroy the object.
        """
        self._post_event(1)


class ExtTransientSeatManagerV1Proxy(Proxy[ExtTransientSeatManagerV1]):
    interface = ExtTransientSeatManagerV1

    @ExtTransientSeatManagerV1.request(
        Argument(ArgumentType.NewId, interface=ExtTransientSeatV1),
    )
    def create(self) -> ExtTransientSeatV1Proxy:
        """Create a transient seat

        Create a new seat that is removed when the client side transient seat
        object is destroyed.

        The actual seat may be removed sooner, in which case the transient seat
        object shall become inert.

        :returns:
            :class:`~pywayland.protocol.ext_transient_seat_v1.ExtTransientSeatV1`
        """
        seat = self._marshal_constructor(0, ExtTransientSeatV1)
        return seat

    @ExtTransientSeatManagerV1.request()
    def destroy(self) -> None:
        """Destroy the manager

        Destroy the manager.

        All objects created by the manager will remain valid until they are
        destroyed themselves.
        """
        self._marshal(1)
        self._destroy()


class ExtTransientSeatV1Proxy(Proxy[ExtTransientSeatV1]):
    interface = ExtTransientSeatV1

    @ExtTransientSeatV1.request()
    def destroy(self) -> None:
        """Destroy transient seat

        When the transient seat object is destroyed by the client, the
        associated seat created by the compositor is also destroyed.
        """
        self._marshal(0)
        self._destroy()


class ExtTransientSeatManagerV1Global(Global[ExtTransientSeatManagerV1]):
    interface = ExtTransientSeatManagerV1


class ExtTransientSeatV1Global(Global[ExtTransientSeatV1]):
    interface = ExtTransientSeatV1


ExtTransientSeatManagerV1._gen_c()
ExtTransientSeatManagerV1.proxy_class = ExtTransientSeatManagerV1Proxy
ExtTransientSeatManagerV1.resource_class = ExtTransientSeatManagerV1Resource
ExtTransientSeatManagerV1.global_class = ExtTransientSeatManagerV1Global


ExtTransientSeatV1._gen_c()
ExtTransientSeatV1.proxy_class = ExtTransientSeatV1Proxy
ExtTransientSeatV1.resource_class = ExtTransientSeatV1Resource
ExtTransientSeatV1.global_class = ExtTransientSeatV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2023-2024 Matthias Klumpp
# Copyright ©      2024 David Edmundson
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlBuffer
from .wayland import WlBufferProxy
from .xdg_shell import XdgToplevel
from .xdg_shell import XdgToplevelProxy


class XdgToplevelIconManagerV1(Interface):
    """Interface to manage toplevel icons

    This interface allows clients to create toplevel window icons and set them
    on toplevel windows to be displayed to the user.
    """

    name = "xdg_toplevel_icon_manager_v1"
    version = 1


class XdgToplevelIconV1(Interface):
    """A toplevel window icon

    This interface defines a toplevel icon. An icon can have a name, and
    multiple buffers. In order to be applied, the icon must have either a name,
    or at least one buffer assigned. Applying an empty icon (with no buffer or
    name) to a toplevel should reset its icon to the default icon.

    It is up to compositor policy whether to prefer using a buffer or loading
    an icon via its name. See 'set_name' and 'add_buffer' for details.
    """

    name = "xdg_toplevel_icon_v1"
    version = 1

    class error(enum.IntEnum):
        invalid_buffer = 1
        immutable = 2
        no_buffer = 3


class XdgToplevelIconManagerV1Resource(Resource[XdgToplevelIconManagerV1]):
    interface = XdgToplevelIconManagerV1

    @XdgToplevelIconManagerV1.event(
        Argument(ArgumentType.Int),
    )
    def icon_size(self, size: int) -> None:
        """Describes a supported & preferred icon size

        This event indicates an icon size the compositor prefers to be
        available if the client has scalable icons and can render to any size.

        When the 'xdg_toplevel_icon_manager_v1' object is created, the
        compositor may send one or more 'icon_size' events to describe the list
        of preferred icon sizes. If the compositor has no size preference, it
        may not send any 'icon_size' event, and it is up to the client to
        decide a suitable icon size.

        A sequence of 'icon_size' events must be finished with a 'done' event.
        If the compositor has no size preferences, it must still send the
        'done' event, without any preceding 'icon_size' events.

        :param size:
            the edge size of the square icon in surface-local coordinates, e.g.
            64
        :type size:
            `ArgumentType.Int`
        """
        self._post_event(0, size)

    @XdgToplevelIconManagerV1.event()
    def done(self) -> None:
        """All information has been sent

        This event is sent after all 'icon_size' events have been sent.
        """
        self._post_event(1)


class XdgToplevelIconV1Resource(Resource[XdgToplevelIconV1]):
    interface = XdgToplevelIconV1


class XdgToplevelIconManagerV1Proxy(Proxy[XdgToplevelIconManagerV1]):
    interface = XdgToplevelIconManagerV1

    @XdgToplevelIconManagerV1.request()
    def destroy(self) -> None:
        """Destroy the toplevel icon manager

        Destroy the toplevel icon manager. This does not destroy objects
        created with the manager.
        """
        self._marshal(0)
        self._destroy()

    @XdgToplevelIconManagerV1.request(
        Argument(ArgumentType.NewId, interface=XdgToplevelIconV1),
    )
    def create_icon(self) -> XdgToplevelIconV1Proxy:
        """Create a new icon instance

        Creates a new icon object. This icon can then be attached to a
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` via the 'set_icon'
        request.

        :returns:
            :class:`~pywayland.protocol.xdg_toplevel_icon_v1.XdgToplevelIconV1`
        """
        id = self._marshal_constructor(1, XdgToplevelIconV1)
        return id

    @XdgToplevelIconManagerV1.request(
        Argument(ArgumentType.Object, interface=XdgToplevel),
        Argument(ArgumentType.Object, interface=XdgToplevelIconV1, nullable=True),
    )
    def set_icon(self, toplevel: XdgToplevelProxy, icon: XdgToplevelIconV1Proxy | None) -> None:
        """Set an icon on a toplevel window

        This request assigns the icon 'icon' to 'toplevel', or clears the
        toplevel icon if 'icon' was null. This state is double-buffered and is
        applied on the next :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` of the toplevel.

        After making this call, the
        :class:`~pywayland.protocol.xdg_toplevel_icon_v1.XdgToplevelIconV1`
        provided as 'icon' can be destroyed by the client without 'toplevel'
        losing its icon. The
        :class:`~pywayland.protocol.xdg_toplevel_icon_v1.XdgToplevelIconV1` is
        immutable from this point, and any future attempts to change it must
        raise the
        ':class:`~pywayland.protocol.xdg_toplevel_icon_v1.XdgToplevelIconV1`.immutable'
        protocol error.

        The compositor must set the toplevel icon from either the pixel data
        the icon provides, or by loading a stock icon using the icon name. See
        the description of 'xdg_toplevel_icon_v1' for details.

        If 'icon' is set to null, the icon of the respective toplevel is reset
        to its default icon (usually the icon of the application, derived from
        its desktop-entry file, or a placeholder icon). If this request is
        passed an icon with no pixel buffers or icon name assigned, the icon
        must be reset just like if 'icon' was null.

        :param toplevel:
            the toplevel to act on
        :type toplevel:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        :param icon:
        :type icon:
            :class:`~pywayland.protocol.xdg_toplevel_icon_v1.XdgToplevelIconV1` or `None`
        """
        self._marshal(2, toplevel, icon)


class XdgToplevelIconV1Proxy(Proxy[XdgToplevelIconV1]):
    interface = XdgToplevelIconV1

    @XdgToplevelIconV1.request()
    def destroy(self) -> None:
        """Destroy the icon object

        Destroys the 'xdg_toplevel_icon_v1' object. The icon must still remain
        set on every toplevel it was assigned to, until the toplevel icon is
        reset explicitly.
        """
        self._marshal(0)
        self._destroy()

    @XdgToplevelIconV1.request(
        Argument(ArgumentType.String),
    )
    def set_name(self, icon_name: str) -> None:
        """Set an icon name

        This request assigns an icon name to this icon. Any previously set name
        is overridden.

        The compositor must resolve 'icon_name' according to the lookup rules
        described in the XDG icon theme specification[1] using the
        environment's current icon theme.

        If the compositor does not support icon names or cannot resolve
        'icon_name' according to the XDG icon theme specification it must fall
        back to using pixel buffer data instead.

        If this request is made after the icon has been assigned to a toplevel
        via 'set_icon', an 'immutable' error must be raised.

        [1]: https://specifications.freedesktop.org/icon-theme-spec/icon-theme-
        spec-latest.html

        :param icon_name:
        :type icon_name:
            `ArgumentType.String`
        """
        self._marshal(1, icon_name)

    @XdgToplevelIconV1.request(
        Argument(ArgumentType.Object, interface=WlBuffer),
        Argument(ArgumentType.Int),
    )
    def add_buffer(self, buffer: WlBufferProxy, scale: int) -> None:
        """Add icon data from a pixel buffer

        This request adds pixel data supplied as
        :class:`~pywayland.protocol.wayland.WlBuffer` to the icon.

        The client should add pixel data for all icon sizes and scales that it
        can provide, or which are explicitly requested by the compositor via
        'icon_size' events on
        :class:`~pywayland.protocol.xdg_toplevel_icon_v1.XdgToplevelIconManagerV1`.

        The :class:`~pywayland.protocol.wayland.WlBuffer` supplying pixel data
        as 'buffer' must be backed by
        :class:`~pywayland.protocol.wayland.WlShm` and must be a square (width
        and height being equal). If any of these buffer requirements are not
        fulfilled, a 'invalid_buffer' error must be raised.

        If this icon instance already has a buffer of the same size and scale
        from a previous 'add_buffer' request, data from the last request
        overrides the preexisting pixel data.

        The :class:`~pywayland.protocol.wayland.WlBuffer` must be kept alive
        for as long as the xdg_toplevel_icon it is associated with is not
        destroyed, otherwise a 'no_buffer' error is raised. The buffer contents
        must not be modified after it was assigned to the icon. As a result,
        the region of the wl_shm_pool's backing storage used for the
        :class:`~pywayland.protocol.wayland.WlBuffer` must not be modified
        after this request is sent. The :func:`WlBuffer.release()
        <pywayland.protocol.wayland.WlBuffer.release>` event is unused.

        If this request is made after the icon has been assigned to a toplevel
        via 'set_icon', an 'immutable' error must be raised.

        :param buffer:
        :type buffer:
            :class:`~pywayland.protocol.wayland.WlBuffer`
        :param scale:
            the scaling factor of the icon, e.g. 1
        :type scale:
            `ArgumentType.Int`
        """
        self._marshal(2, buffer, scale)


class XdgToplevelIconManagerV1Global(Global[XdgToplevelIconManagerV1]):
    interface = XdgToplevelIconManagerV1


class XdgToplevelIconV1Global(Global[XdgToplevelIconV1]):
    interface = XdgToplevelIconV1


XdgToplevelIconManagerV1._gen_c()
XdgToplevelIconManagerV1.proxy_class = XdgToplevelIconManagerV1Proxy
XdgToplevelIconManagerV1.resource_class = XdgToplevelIconManagerV1Resource
XdgToplevelIconManagerV1.global_class = XdgToplevelIconManagerV1Global


XdgToplevelIconV1._gen_c()
XdgToplevelIconV1.proxy_class = XdgToplevelIconV1Proxy
XdgToplevelIconV1.resource_class = XdgToplevelIconV1Resource
XdgToplevelIconV1.global_class = XdgToplevelIconV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2019 Sebastian Wick
# Copyright 2019 Erwin Burema
# Copyright 2020 AMD
# Copyright 2020-2024 Collabora, Ltd.
# Copyright 2024 Xaver Hugl
# Copyright 2022-2025 Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlOutput
from .wayland import WlOutputProxy
from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class WpColorManagementOutputV1(Interface):
    """Output color properties

    A :class:`WpColorManagementOutputV1` describes the color properties of an
    output.

    The :class:`WpColorManagementOutputV1` is associated with the
    :class:`~pywayland.protocol.wayland.WlOutput` global underlying the
    :class:`~pywayland.protocol.wayland.WlOutput` object. Therefore the client
    destroying the :class:`~pywayland.protocol.wayland.WlOutput` object has no
    impact, but the compositor removing the output global makes the
    :class:`WpColorManagementOutputV1` object inert.
    """

    name = "wp_color_management_output_v1"
    version = 3


class WpColorManagementSurfaceFeedbackV1(Interface):
    """Color management extension to a surface

    A :class:`WpColorManagementSurfaceFeedbackV1` allows the client to get the
    preferred image description of a surface.

    If the :class:`~pywayland.protocol.wayland.WlSurface` associated with this
    object is destroyed, the :class:`WpColorManagementSurfaceFeedbackV1` object
    becomes inert.
    """

    name = "wp_color_management_surface_feedback_v1"
    version = 3

    class error(enum.IntEnum):
        inert = 0
        unsupported_feature = 1


class WpColorManagementSurfaceV1(Interface):
    """Color management extension to a surface

    A :class:`WpColorManagementSurfaceV1` allows the client to set the color
    space and HDR properties of a surface.

    If the :class:`~pywayland.protocol.wayland.WlSurface` associated with the
    :class:`WpColorManagementSurfaceV1` is destroyed, the
    :class:`WpColorManagementSurfaceV1` object becomes inert.
    """

    name = "wp_color_management_surface_v1"
    version = 3

    class error(enum.IntEnum):
        render_intent = 0
        image_description = 1
        inert = 2


class WpColorManagerV1(Interface):
    """Color manager singleton

    A singleton global interface used for getting color management extensions
    for :class:`~pywayland.protocol.wayland.WlSurface` and
    :class:`~pywayland.protocol.wayland.WlOutput` objects, and for creating
    client defined image description objects. The extension interfaces allow
    getting the image description of outputs and setting the image description
    of surfaces.

    Compositors should never remove this global.
    """

    name = "wp_color_manager_v1"
    version = 3

    class error(enum.IntEnum):
        unsupported_feature = 0
        surface_exists = 1

    class render_intent(enum.IntEnum):
        perceptual = 0
        relative = 1
        saturation = 2
        absolute = 3
        relative_bpc = 4
        absolute_no_adaptation = 5

    class feature(enum.IntEnum):
        icc_v2_v4 = 0
        parametric = 1
        set_primaries = 2
        set_tf_power = 3
        set_luminances = 4
        set_mastering_display_primaries = 5
        extended_target_volume = 6
        windows_scrgb = 7
        windows_bt2100 = 8

    class primaries(enum.IntEnum):
        srgb = 1
        pal_m = 2
        pal = 3
        ntsc = 4
        generic_film = 5
        bt2020 = 6
        cie1931_xyz = 7
        dci_p3 = 8
        display_p3 = 9
        adobe_rgb = 10

    class transfer_function(enum.IntEnum):
        bt1886 = 1
        gamma22 = 2
        gamma28 = 3
        st240 = 4
        ext_linear = 5
        log_100 = 6
        log_316 = 7
        xvycc = 8
        srgb = 9
        ext_srgb = 10
        st2084_pq = 11
        st428 = 12
        hlg = 13
        compound_power_2_4 = 14


class WpImageDescriptionCreatorIccV1(Interface):
    """Holder of image description icc information

    This type of object is used for collecting all the information required to
    create a
    :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
    object from an ICC file. A complete set of required parameters consists of
    these properties: - ICC file

    Each required property must be set exactly once if the client is to create
    an image description. The set requests verify that a property was not
    already set. The create request verifies that all required properties are
    set. There may be several alternative requests for setting each property,
    and in that case the client must choose one of them.

    Once all properties have been set, the create request must be used to
    create the image description object, destroying the creator in the process.

    The link between a pixel value (a device value in ICC) and its respective
    colorimetry is defined by the details of the particular ICC profile. Those
    details also determine when colorimetry becomes undefined.
    """

    name = "wp_image_description_creator_icc_v1"
    version = 3

    class error(enum.IntEnum):
        incomplete_set = 0
        already_set = 1
        bad_fd = 2
        bad_size = 3
        out_of_file = 4


class WpImageDescriptionCreatorParamsV1(Interface):
    """Holder of image description parameters

    This type of object is used for collecting all the parameters required to
    create a
    :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
    object. A complete set of required parameters consists of these properties:
    - transfer characteristic function (tf) - chromaticities of primaries and
    white point (primary color volume)

    The following properties are optional and have a well-defined default if
    not explicitly set: - primary color volume luminance range - reference
    white luminance level - mastering display primaries and white point (target
    color volume) - mastering luminance range

    The following properties are optional and will be ignored if not explicitly
    set: - maximum content light level - maximum frame-average light level

    Each required property must be set exactly once if the client is to create
    an image description. The set requests verify that a property was not
    already set. The create request verifies that all required properties are
    set. There may be several alternative requests for setting each property,
    and in that case the client must choose one of them.

    Once all properties have been set, the create request must be used to
    create the image description object, destroying the creator in the process.

    A viewer, who is viewing the display defined by the resulting image
    description (the viewing environment included), is assumed to be fully
    adapted to the primary color volume's white point.

    Any of the following conditions will cause the colorimetry of a pixel to
    become undefined: - Values outside of the defined range of the transfer
    characteristic. - Tristimulus that exceeds the target color volume. - If
    extended_target_volume is not supported: tristimulus that exceeds the
    primary color volume.

    The closest correspondence to an image description created through this
    interface is the Display class of profiles in ICC.
    """

    name = "wp_image_description_creator_params_v1"
    version = 3

    class error(enum.IntEnum):
        incomplete_set = 0
        already_set = 1
        unsupported_feature = 2
        invalid_tf = 3
        invalid_primaries_named = 4
        invalid_luminance = 5


class WpImageDescriptionInfoV1(Interface):
    """Colorimetric image description information

    Sends all matching events describing an image description object exactly
    once and finally sends the 'done' event.

    This means - if the image description is parametric, it must send   -
    primaries   - named_primaries, if applicable   - at least one of tf_power
    and tf_named, as applicable   - luminances   - target_primaries   -
    target_luminance - if the image description is parametric, it may send, if
    applicable,   - target_max_cll   - target_max_fall - if the image
    description contains an ICC profile, it must send the   icc_file event

    Once a :class:`WpImageDescriptionInfoV1` object has delivered a 'done'
    event it is automatically destroyed.

    Every :class:`WpImageDescriptionInfoV1` created from the same
    :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1` shall
    always return the exact same data.
    """

    name = "wp_image_description_info_v1"
    version = 3


class WpImageDescriptionReferenceV1(Interface):
    """Reference to an image description

    This object is a reference to an image description. This interface is
    frozen at version 1 to allow other protocols to create
    :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
    objects.

    The :func:`WpColorManagerV1.get_image_description()
    <pywayland.protocol.color_management_v1.WpColorManagerV1.get_image_description>`
    request can be used to retrieve the underlying image description.
    """

    name = "wp_image_description_reference_v1"
    version = 1


class WpImageDescriptionV1(Interface):
    """Colorimetric image description

    An image description carries information about the pixel color encoding and
    its intended display and viewing environment. The image description is
    attached to a :class:`~pywayland.protocol.wayland.WlSurface` via
    :func:`WpColorManagementSurfaceV1.set_image_description()
    <pywayland.protocol.color_management_v1.WpColorManagementSurfaceV1.set_image_description>`.
    A compositor can use this information to decode pixel values into
    colorimetrically meaningful quantities, which allows the compositor to
    transform the surface contents to become suitable for various displays and
    viewing environments.

    Note, that the :class:`WpImageDescriptionV1` object is not ready to be used
    immediately after creation. The object eventually delivers either the
    'ready' or the 'failed' event, specified in all requests creating it. The
    object is deemed "ready" after receiving the 'ready' event.

    An object which is not ready is illegal to use, it can only be destroyed.
    Any other request in this interface shall result in the 'not_ready'
    protocol error. Attempts to use an object which is not ready through other
    interfaces shall raise protocol errors defined there.

    Once created and regardless of how it was created, a
    :class:`WpImageDescriptionV1` object always refers to one fixed image
    description. It cannot change after creation.
    """

    name = "wp_image_description_v1"
    version = 3

    class error(enum.IntEnum):
        not_ready = 0
        no_information = 1

    class cause(enum.IntEnum):
        low_version = 0
        unsupported = 1
        operating_system = 2
        no_output = 3


class WpColorManagementOutputV1Resource(Resource[WpColorManagementOutputV1]):
    interface = WpColorManagementOutputV1

    @WpColorManagementOutputV1.event()
    def image_description_changed(self) -> None:
        """Image description changed

        This event is sent whenever the image description of the output
        changed, followed by one :func:`WlOutput.done()
        <pywayland.protocol.wayland.WlOutput.done>` event common to output
        events across all extensions.

        If the client wants to use the updated image description, it needs to
        do get_image_description again, because image description objects are
        immutable.
        """
        self._post_event(0)


class WpColorManagementSurfaceFeedbackV1Resource(Resource[WpColorManagementSurfaceFeedbackV1]):
    interface = WpColorManagementSurfaceFeedbackV1

    @WpColorManagementSurfaceFeedbackV1.event(
        Argument(ArgumentType.Uint),
    )
    def preferred_changed(self, identity: int) -> None:
        """The preferred image description changed (32-bit)

        Starting from interface version 2, 'preferred_changed2' is sent instead
        of this event. See the 'preferred_changed2' event for the definition.

        :param identity:
            the 32-bit image description id number
        :type identity:
            `ArgumentType.Uint`
        """
        self._post_event(0, identity)

    @WpColorManagementSurfaceFeedbackV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        version=2,
    )
    def preferred_changed2(self, identity_hi: int, identity_lo: int) -> None:
        """The preferred image description changed

        The preferred image description is the one which likely has the most
        performance and/or quality benefits for the compositor if used by the
        client for its :class:`~pywayland.protocol.wayland.WlSurface` contents.
        This event is sent whenever the compositor changes the wl_surface's
        preferred image description.

        This event sends the identity of the new preferred state as the
        argument, so clients who are aware of the image description already can
        reuse it. Otherwise, if the client client wants to know what the
        preferred image description is, it shall use the get_preferred request.

        The preferred image description is not automatically used for anything.
        It is only a hint, and clients may set any valid image description with
        set_image_description, but there might be performance and color
        accuracy improvements by providing the
        :class:`~pywayland.protocol.wayland.WlSurface` contents in the
        preferred image description. Therefore clients that can, should render
        according to the preferred image description

        :param identity_hi:
            high 32 bits of the 64-bit image description id number
        :type identity_hi:
            `ArgumentType.Uint`
        :param identity_lo:
            low 32 bits of the 64-bit image description id number
        :type identity_lo:
            `ArgumentType.Uint`
        """
        self._post_event(1, identity_hi, identity_lo)


class WpColorManagementSurfaceV1Resource(Resource[WpColorManagementSurfaceV1]):
    interface = WpColorManagementSurfaceV1


class WpColorManagerV1Resource(Resource[WpColorManagerV1]):
    interface = WpColorManagerV1

    @WpColorManagerV1.event(
        Argument(ArgumentType.Uint),
    )
    def supported_intent(self, render_intent: int) -> None:
        """Supported rendering intent

        When this object is created, it shall immediately send this event once
        for each rendering intent the compositor supports.

        A compositor must not advertise intents that are deprecated in the
        bound version of the interface.

        :param render_intent:
            rendering intent
        :type render_intent:
            `ArgumentType.Uint`
        """
        self._post_event(0, render_intent)

    @WpColorManagerV1.event(
        Argument(ArgumentType.Uint),
    )
    def supported_feature(self, feature: int) -> None:
        """Supported features

        When this object is created, it shall immediately send this event once
        for each compositor supported feature listed in the enumeration.

        A compositor must not advertise features that are deprecated in the
        bound version of the interface.

        :param feature:
            supported feature
        :type feature:
            `ArgumentType.Uint`
        """
        self._post_event(1, feature)

    @WpColorManagerV1.event(
        Argument(ArgumentType.Uint),
    )
    def supported_tf_named(self, tf: int) -> None:
        """Supported named transfer characteristic

        When this object is created, it shall immediately send this event once
        for each named transfer function the compositor supports with the
        parametric image description creator.

        A compositor must not advertise transfer functions that are deprecated
        in the bound version of the interface.

        :param tf:
            Named transfer function
        :type tf:
            `ArgumentType.Uint`
        """
        self._post_event(2, tf)

    @WpColorManagerV1.event(
        Argument(ArgumentType.Uint),
    )
    def supported_primaries_named(self, primaries: int) -> None:
        """Supported named primaries

        When this object is created, it shall immediately send this event once
        for each named set of primaries the compositor supports with the
        parametric image description creator.

        A compositor must not advertise names that are deprecated in the bound
        version of the interface.

        :param primaries:
            Named color primaries
        :type primaries:
            `ArgumentType.Uint`
        """
        self._post_event(3, primaries)

    @WpColorManagerV1.event()
    def done(self) -> None:
        """All features have been sent

        This event is sent when all supported rendering intents, features,
        transfer functions and named primaries have been sent.
        """
        self._post_event(4)


class WpImageDescriptionCreatorIccV1Resource(Resource[WpImageDescriptionCreatorIccV1]):
    interface = WpImageDescriptionCreatorIccV1


class WpImageDescriptionCreatorParamsV1Resource(Resource[WpImageDescriptionCreatorParamsV1]):
    interface = WpImageDescriptionCreatorParamsV1


class WpImageDescriptionInfoV1Resource(Resource[WpImageDescriptionInfoV1]):
    interface = WpImageDescriptionInfoV1

    @WpImageDescriptionInfoV1.event()
    def done(self) -> None:
        """End of information

        Signals the end of information events and destroys the object.
        """
        self._post_event(0)

    @WpImageDescriptionInfoV1.event(
        Argument(ArgumentType.FileDescriptor),
        Argument(ArgumentType.Uint),
    )
    def icc_file(self, icc: int, icc_size: int) -> None:
        """Icc profile matching the image description

        The icc argument provides a file descriptor to the client which may be
        memory-mapped to provide the ICC profile matching the image
        description. The fd is read-only, and if mapped then it must be mapped
        with MAP_PRIVATE by the client.

        The ICC profile version and other details are determined by the
        compositor. There is no provision for a client to ask for a specific
        kind of a profile.

        :param icc:
            ICC profile file descriptor
        :type icc:
            `ArgumentType.FileDescriptor`
        :param icc_size:
            ICC profile size, in bytes
        :type icc_size:
            `ArgumentType.Uint`
        """
        self._post_event(1, icc, icc_size)

    @WpImageDescriptionInfoV1.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def primaries(self, r_x: int, r_y: int, g_x: int, g_y: int, b_x: int, b_y: int, w_x: int, w_y: int) -> None:
        """Primaries as chromaticity coordinates

        Delivers the primary color volume primaries and white point using CIE
        1931 xy chromaticity coordinates.

        Each coordinate value is multiplied by 1 million to get the argument
        value to carry precision of 6 decimals.

        :param r_x:
            Red x * 1M
        :type r_x:
            `ArgumentType.Int`
        :param r_y:
            Red y * 1M
        :type r_y:
            `ArgumentType.Int`
        :param g_x:
            Green x * 1M
        :type g_x:
            `ArgumentType.Int`
        :param g_y:
            Green y * 1M
        :type g_y:
            `ArgumentType.Int`
        :param b_x:
            Blue x * 1M
        :type b_x:
            `ArgumentType.Int`
        :param b_y:
            Blue y * 1M
        :type b_y:
            `ArgumentType.Int`
        :param w_x:
            White x * 1M
        :type w_x:
            `ArgumentType.Int`
        :param w_y:
            White y * 1M
        :type w_y:
            `ArgumentType.Int`
        """
        self._post_event(2, r_x, r_y, g_x, g_y, b_x, b_y, w_x, w_y)

    @WpImageDescriptionInfoV1.event(
        Argument(ArgumentType.Uint),
    )
    def primaries_named(self, primaries: int) -> None:
        """Named primaries

        Delivers the primary color volume primaries and white point using an
        explicitly enumerated named set.

        :param primaries:
            named primaries
        :type primaries:
            `ArgumentType.Uint`
        """
        self._post_event(3, primaries)

    @WpImageDescriptionInfoV1.event(
        Argument(ArgumentType.Uint),
    )
    def tf_power(self, eexp: int) -> None:
        """Transfer characteristic as a power curve

        The color component transfer characteristic of this image description
        is a pure power curve. This event provides the exponent of the power
        function. This curve represents the conversion from electrical to
        optical pixel or color values.

        The curve exponent has been multiplied by 10000 to get the argument
        eexp value to carry the precision of 4 decimals.

        :param eexp:
            the exponent * 10000
        :type eexp:
            `ArgumentType.Uint`
        """
        self._post_event(4, eexp)

    @WpImageDescriptionInfoV1.event(
        Argument(ArgumentType.Uint),
    )
    def tf_named(self, tf: int) -> None:
        """Named transfer characteristic

        Delivers the transfer characteristic using an explicitly enumerated
        named function.

        :param tf:
            named transfer function
        :type tf:
            `ArgumentType.Uint`
        """
        self._post_event(5, tf)

    @WpImageDescriptionInfoV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def luminances(self, min_lum: int, max_lum: int, reference_lum: int) -> None:
        """Primary color volume luminance range and reference white

        Delivers the primary color volume luminance range and the reference
        white luminance level. These values include the minimum display
        emission and ambient flare luminances, assumed to be optically additive
        and have the chromaticity of the primary color volume white point.

        The minimum luminance is multiplied by 10000 to get the argument
        'min_lum' value and carries precision of 4 decimals. The maximum
        luminance and reference white luminance values are unscaled.

        :param min_lum:
            minimum luminance (cd/m²) * 10000
        :type min_lum:
            `ArgumentType.Uint`
        :param max_lum:
            maximum luminance (cd/m²)
        :type max_lum:
            `ArgumentType.Uint`
        :param reference_lum:
            reference white luminance (cd/m²)
        :type reference_lum:
            `ArgumentType.Uint`
        """
        self._post_event(6, min_lum, max_lum, reference_lum)

    @WpImageDescriptionInfoV1.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def target_primaries(self, r_x: int, r_y: int, g_x: int, g_y: int, b_x: int, b_y: int, w_x: int, w_y: int) -> None:
        """Target primaries as chromaticity coordinates

        Provides the color primaries and white point of the target color volume
        using CIE 1931 xy chromaticity coordinates. This is compatible with the
        SMPTE ST 2086 definition of HDR static metadata for mastering displays.

        While primary color volume is about how color is encoded, the target
        color volume is the actually displayable color volume.

        Each coordinate value is multiplied by 1 million to get the argument
        value to carry precision of 6 decimals.

        :param r_x:
            Red x * 1M
        :type r_x:
            `ArgumentType.Int`
        :param r_y:
            Red y * 1M
        :type r_y:
            `ArgumentType.Int`
        :param g_x:
            Green x * 1M
        :type g_x:
            `ArgumentType.Int`
        :param g_y:
            Green y * 1M
        :type g_y:
            `ArgumentType.Int`
        :param b_x:
            Blue x * 1M
        :type b_x:
            `ArgumentType.Int`
        :param b_y:
            Blue y * 1M
        :type b_y:
            `ArgumentType.Int`
        :param w_x:
            White x * 1M
        :type w_x:
            `ArgumentType.Int`
        :param w_y:
            White y * 1M
        :type w_y:
            `ArgumentType.Int`
        """
        self._post_event(7, r_x, r_y, g_x, g_y, b_x, b_y, w_x, w_y)

    @WpImageDescriptionInfoV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def target_luminance(self, min_lum: int, max_lum: int) -> None:
        """Target luminance range

        Provides the luminance range that the image description is targeting as
        the minimum and maximum absolute luminance L. These values include the
        minimum display emission and ambient flare luminances, assumed to be
        optically additive and have the chromaticity of the primary color
        volume white point. This should be compatible with the SMPTE ST 2086
        definition of HDR static metadata.

        This luminance range is only theoretical and may not correspond to the
        luminance of light emitted on an actual display.

        Min L value is multiplied by 10000 to get the argument min_lum value
        and carry precision of 4 decimals. Max L value is unscaled for max_lum.

        :param min_lum:
            min L (cd/m²) * 10000
        :type min_lum:
            `ArgumentType.Uint`
        :param max_lum:
            max L (cd/m²)
        :type max_lum:
            `ArgumentType.Uint`
        """
        self._post_event(8, min_lum, max_lum)

    @WpImageDescriptionInfoV1.event(
        Argument(ArgumentType.Uint),
    )
    def target_max_cll(self, max_cll: int) -> None:
        """Target maximum content light level

        Provides the targeted max_cll of the image description. max_cll is
        defined by CTA-861-H.

        This luminance is only theoretical and may not correspond to the
        luminance of light emitted on an actual display.

        :param max_cll:
            Maximum content light-level (cd/m²)
        :type max_cll:
            `ArgumentType.Uint`
        """
        self._post_event(9, max_cll)

    @WpImageDescriptionInfoV1.event(
        Argument(ArgumentType.Uint),
    )
    def target_max_fall(self, max_fall: int) -> None:
        """Target maximum frame-average light level

        Provides the targeted max_fall of the image description. max_fall is
        defined by CTA-861-H.

        This luminance is only theoretical and may not correspond to the
        luminance of light emitted on an actual display.

        :param max_fall:
            Maximum frame-average light level (cd/m²)
        :type max_fall:
            `ArgumentType.Uint`
        """
        self._post_event(10, max_fall)


class WpImageDescriptionReferenceV1Resource(Resource[WpImageDescriptionReferenceV1]):
    interface = WpImageDescriptionReferenceV1


class WpImageDescriptionV1Resource(Resource[WpImageDescriptionV1]):
    interface = WpImageDescriptionV1

    @WpImageDescriptionV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
    )
    def failed(self, cause: int, msg: str) -> None:
        """Graceful error on creating the image description

        If creating a :class:`WpImageDescriptionV1` object fails for a reason
        that is not defined as a protocol error, this event is sent.

        The requests that create image description objects define whether and
        when this can occur. Only such creation requests can trigger this
        event. This event cannot be triggered after the image description was
        successfully formed.

        Once this event has been sent, the :class:`WpImageDescriptionV1` object
        will never become ready and it can only be destroyed.

        :param cause:
            generic reason
        :type cause:
            `ArgumentType.Uint`
        :param msg:
            ad hoc human-readable explanation
        :type msg:
            `ArgumentType.String`
        """
        self._post_event(0, cause, msg)

    @WpImageDescriptionV1.event(
        Argument(ArgumentType.Uint),
    )
    def ready(self, identity: int) -> None:
        """The object is ready to be used (32-bit)

        Starting from interface version 2, the 'ready2' event is sent instead
        of this event.

        For the definition of this event, see the 'ready2' event. The
        difference to this event is as follows.

        The id number is valid only as long as the protocol object is alive. If
        all protocol objects referring to the same image description record are
        destroyed, the id number may be recycled for a different image
        description record.

        :param identity:
            the 32-bit image description id number
        :type identity:
            `ArgumentType.Uint`
        """
        self._post_event(1, identity)

    @WpImageDescriptionV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        version=2,
    )
    def ready2(self, identity_hi: int, identity_lo: int) -> None:
        """The object is ready to be used

        Once this event has been sent, the :class:`WpImageDescriptionV1` object
        is deemed "ready". Ready objects can be used to send requests and can
        be used through other interfaces.

        Every ready :class:`WpImageDescriptionV1` protocol object refers to an
        underlying image description record in the compositor. Multiple
        protocol objects may end up referring to the same record. Clients may
        identify these "copies" by comparing their id numbers: if the numbers
        from two protocol objects are identical, the protocol objects refer to
        the same image description record. Two different image description
        records cannot have the same id number simultaneously. The id number
        does not change during the lifetime of the image description record.

        Image description id number is not a protocol object id. Zero is
        reserved as an invalid id number. It shall not be possible for a client
        to refer to an image description by its id number in protocol. The id
        numbers might not be portable between Wayland connections. A compositor
        shall not send an invalid id number.

        Compositors must not recycle image description id numbers.

        This identity allows clients to de-duplicate image description records
        and avoid get_information request if they already have the image
        description information.

        :param identity_hi:
            high 32 bits of the 64-bit image description id number
        :type identity_hi:
            `ArgumentType.Uint`
        :param identity_lo:
            low 32 bits of the 64-bit image description id number
        :type identity_lo:
            `ArgumentType.Uint`
        """
        self._post_event(2, identity_hi, identity_lo)


class WpColorManagementOutputV1Proxy(Proxy[WpColorManagementOutputV1]):
    interface = WpColorManagementOutputV1

    @WpColorManagementOutputV1.request()
    def destroy(self) -> None:
        """Destroy the color management output

        Destroy the color :class:`WpColorManagementOutputV1` object. This does
        not affect any remaining protocol objects.
        """
        self._marshal(0)
        self._destroy()

    @WpColorManagementOutputV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionV1),
    )
    def get_image_description(self) -> WpImageDescriptionV1Proxy:
        """Get the image description of the output

        This creates a new
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        object for the current image description of the output. There always is
        exactly one image description active for an output so the client should
        destroy the image description created by earlier invocations of this
        request. This request is usually sent as a reaction to the
        image_description_changed event or when creating a
        :class:`WpColorManagementOutputV1` object.

        The image description of an output represents the color encoding the
        output expects. There might be performance and power advantages, as
        well as improved color reproduction, if a content update matches the
        image description of the output it is being shown on. If a content
        update is shown on any other output than the one it matches the image
        description of, then the color reproduction on those outputs might be
        considerably worse.

        The created
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        object preserves the image description of the output from the time the
        object was created.

        The resulting image description object allows get_information request.

        If this protocol object is inert, the resulting image description
        object shall immediately deliver the
        :func:`WpImageDescriptionV1.failed()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.failed>`
        event with the no_output cause.

        If the interface version is inadequate for the output's image
        description, meaning that the client does not support all the events
        needed to deliver the crucial information, the resulting image
        description object shall immediately deliver the
        :func:`WpImageDescriptionV1.failed()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.failed>`
        event with the low_version cause.

        Otherwise the object shall immediately deliver the ready event.

        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        """
        image_description = self._marshal_constructor(1, WpImageDescriptionV1)
        return image_description


class WpColorManagementSurfaceFeedbackV1Proxy(Proxy[WpColorManagementSurfaceFeedbackV1]):
    interface = WpColorManagementSurfaceFeedbackV1

    @WpColorManagementSurfaceFeedbackV1.request()
    def destroy(self) -> None:
        """Destroy the color management interface for a surface

        Destroy the :class:`WpColorManagementSurfaceFeedbackV1` object.
        """
        self._marshal(0)
        self._destroy()

    @WpColorManagementSurfaceFeedbackV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionV1),
    )
    def get_preferred(self) -> WpImageDescriptionV1Proxy:
        """Get the preferred image description

        If this protocol object is inert, the protocol error inert is raised.

        The preferred image description represents the compositor's preferred
        color encoding for this :class:`~pywayland.protocol.wayland.WlSurface`
        at the current time. There might be performance and power advantages,
        as well as improved color reproduction, if the image description of a
        content update matches the preferred image description.

        This creates a new
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        object for the currently preferred image description for the
        :class:`~pywayland.protocol.wayland.WlSurface`. The client should stop
        using and destroy the image descriptions created by earlier invocations
        of this request for the associated
        :class:`~pywayland.protocol.wayland.WlSurface`. This request is usually
        sent as a reaction to the preferred_changed event or when creating a
        :class:`WpColorManagementSurfaceFeedbackV1` object if the client is
        capable of adapting to image descriptions.

        The created
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        object preserves the preferred image description of the
        :class:`~pywayland.protocol.wayland.WlSurface` from the time the object
        was created.

        The resulting image description object allows get_information request.

        If the image description is parametric, the client should set it on its
        :class:`~pywayland.protocol.wayland.WlSurface` only if the image
        description is an exact match with the client content. Particularly if
        everything else matches, but the target color volume is greater than
        what the client needs, the client should create its own parameric image
        description with its exact parameters.

        If the interface version is inadequate for the preferred image
        description, meaning that the client does not support all the events
        needed to deliver the crucial information, the resulting image
        description object shall immediately deliver the
        :func:`WpImageDescriptionV1.failed()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.failed>`
        event with the low_version cause, otherwise the object shall
        immediately deliver the ready event.

        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        """
        image_description = self._marshal_constructor(1, WpImageDescriptionV1)
        return image_description

    @WpColorManagementSurfaceFeedbackV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionV1),
    )
    def get_preferred_parametric(self) -> WpImageDescriptionV1Proxy:
        """Get the preferred image description

        The same description as for get_preferred applies, except the returned
        image description is guaranteed to be parametric. This is meant for
        clients that can only deal with parametric image descriptions.

        If the compositor doesn't support parametric image descriptions, the
        unsupported_feature error is emitted.

        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        """
        image_description = self._marshal_constructor(2, WpImageDescriptionV1)
        return image_description


class WpColorManagementSurfaceV1Proxy(Proxy[WpColorManagementSurfaceV1]):
    interface = WpColorManagementSurfaceV1

    @WpColorManagementSurfaceV1.request()
    def destroy(self) -> None:
        """Destroy the color management interface for a surface

        Destroy the :class:`WpColorManagementSurfaceV1` object and do the same
        as unset_image_description.
        """
        self._marshal(0)
        self._destroy()

    @WpColorManagementSurfaceV1.request(
        Argument(ArgumentType.Object, interface=WpImageDescriptionV1),
        Argument(ArgumentType.Uint),
    )
    def set_image_description(self, image_description: WpImageDescriptionV1Proxy, render_intent: int) -> None:
        """Set the surface image description

        If this protocol object is inert, the protocol error inert is raised.

        Set the image description of the underlying surface. The image
        description and rendering intent are double-buffered state, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        It is the client's responsibility to understand the image description
        it sets on a surface, and to provide content that matches that image
        description. Compositors might convert images to match their own or any
        other image descriptions.

        Image descriptions which are not ready (see
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`)
        are forbidden in this request, and in such case the protocol error
        image_description is raised.

        All image descriptions which are ready (see
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`)
        are allowed and must always be accepted by the compositor.

        When an image description is set on a surface, it establishes an
        explicit link between surface pixel values and surface colorimetry.
        This link may be undefined for some pixel values, see the image
        description creator interfaces for the conditions. Non-finite floating-
        point values (NaN, Inf) always have an undefined colorimetry.

        A rendering intent provides the client's preference on how surface
        colorimetry should be mapped to each output. The render_intent value
        must be one advertised by the compositor with
        :func:`WpColorManagerV1.render_intent()
        <pywayland.protocol.color_management_v1.WpColorManagerV1.render_intent>`
        event, otherwise the protocol error render_intent is raised.

        By default, a surface does not have an associated image description nor
        a rendering intent. The handling of color on such surfaces is
        compositor implementation defined. Compositors should handle such
        surfaces as sRGB, but may handle them differently if they have specific
        requirements.

        Setting the image description has copy semantics; after this request,
        the image description can be immediately destroyed without affecting
        the pending state of the surface.

        :param image_description:
        :type image_description:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        :param render_intent:
            rendering intent
        :type render_intent:
            `ArgumentType.Uint`
        """
        self._marshal(1, image_description, render_intent)

    @WpColorManagementSurfaceV1.request()
    def unset_image_description(self) -> None:
        """Remove the surface image description

        If this protocol object is inert, the protocol error inert is raised.

        This request removes any image description from the surface. See
        set_image_description for how a compositor handles a surface without an
        image description. This is double-buffered state, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.
        """
        self._marshal(2)


class WpColorManagerV1Proxy(Proxy[WpColorManagerV1]):
    interface = WpColorManagerV1

    @WpColorManagerV1.request()
    def destroy(self) -> None:
        """Destroy the color manager

        Destroy the :class:`WpColorManagerV1` object. This does not affect any
        other objects in any way.
        """
        self._marshal(0)
        self._destroy()

    @WpColorManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpColorManagementOutputV1),
        Argument(ArgumentType.Object, interface=WlOutput),
    )
    def get_output(self, output: WlOutputProxy) -> WpColorManagementOutputV1Proxy:
        """Create a color management interface for a :class:`~pywayland.protocol.wayland.WlOutput`

        This creates a new
        :class:`~pywayland.protocol.color_management_v1.WpColorManagementOutputV1`
        object for the given :class:`~pywayland.protocol.wayland.WlOutput`.

        See the
        :class:`~pywayland.protocol.color_management_v1.WpColorManagementOutputV1`
        interface for more details.

        :param output:
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput`
        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpColorManagementOutputV1`
        """
        id = self._marshal_constructor(1, WpColorManagementOutputV1, output)
        return id

    @WpColorManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpColorManagementSurfaceV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_surface(self, surface: WlSurfaceProxy) -> WpColorManagementSurfaceV1Proxy:
        """Create a color management interface for a :class:`~pywayland.protocol.wayland.WlSurface`

        If a
        :class:`~pywayland.protocol.color_management_v1.WpColorManagementSurfaceV1`
        object already exists for the given
        :class:`~pywayland.protocol.wayland.WlSurface`, the protocol error
        surface_exists is raised.

        This creates a new color
        :class:`~pywayland.protocol.color_management_v1.WpColorManagementSurfaceV1`
        object for the given :class:`~pywayland.protocol.wayland.WlSurface`.

        See the
        :class:`~pywayland.protocol.color_management_v1.WpColorManagementSurfaceV1`
        interface for more details.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpColorManagementSurfaceV1`
        """
        id = self._marshal_constructor(2, WpColorManagementSurfaceV1, surface)
        return id

    @WpColorManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpColorManagementSurfaceFeedbackV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_surface_feedback(self, surface: WlSurfaceProxy) -> WpColorManagementSurfaceFeedbackV1Proxy:
        """Create a color management feedback interface

        This creates a new color
        :class:`~pywayland.protocol.color_management_v1.WpColorManagementSurfaceFeedbackV1`
        object for the given :class:`~pywayland.protocol.wayland.WlSurface`.

        See the
        :class:`~pywayland.protocol.color_management_v1.WpColorManagementSurfaceFeedbackV1`
        interface for more details.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpColorManagementSurfaceFeedbackV1`
        """
        id = self._marshal_constructor(3, WpColorManagementSurfaceFeedbackV1, surface)
        return id

    @WpColorManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionCreatorIccV1),
    )
    def create_icc_creator(self) -> WpImageDescriptionCreatorIccV1Proxy:
        """Make a new icc-based image description creator object

        Makes a new ICC-based image description creator object with all
        properties initially unset. The client can then use the object's
        interface to define all the required properties for an image
        description and finally create a
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        object.

        This request can be used when the compositor advertises
        :func:`WpColorManagerV1.feature()`.icc_v2_v4. Otherwise this request
        raises the protocol error unsupported_feature.

        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionCreatorIccV1`
            -- the new creator object
        """
        obj = self._marshal_constructor(4, WpImageDescriptionCreatorIccV1)
        return obj

    @WpColorManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionCreatorParamsV1),
    )
    def create_parametric_creator(self) -> WpImageDescriptionCreatorParamsV1Proxy:
        """Make a new parametric image description creator object

        Makes a new parametric image description creator object with all
        properties initially unset. The client can then use the object's
        interface to define all the required properties for an image
        description and finally create a
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        object.

        This request can be used when the compositor advertises
        :func:`WpColorManagerV1.feature()`.parametric. Otherwise this request
        raises the protocol error unsupported_feature.

        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionCreatorParamsV1`
            -- the new creator object
        """
        obj = self._marshal_constructor(5, WpImageDescriptionCreatorParamsV1)
        return obj

    @WpColorManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionV1),
    )
    def create_windows_scrgb(self) -> WpImageDescriptionV1Proxy:
        """Create windows-scrgb image description object

        This creates a pre-defined image description for the so-called Windows-
        scRGB stimulus encoding. This comes from the Windows 10 handling of its
        own definition of an scRGB color space for an HDR screen driven in
        BT.2100/PQ signalling mode.

        Windows-scRGB uses sRGB (BT.709) color primaries and white point. The
        transfer characteristic is extended linear.

        The nominal color channel value range is extended, meaning it includes
        negative and greater than 1.0 values. Negative values are used to
        escape the sRGB color gamut boundaries. To make use of the extended
        range, the client needs to use a pixel format that can represent those
        values, e.g. floating-point 16 bits per channel.

        Nominal color value R=G=B=0.0 corresponds to BT.2100/PQ system 0 cd/m²,
        and R=G=B=1.0 corresponds to BT.2100/PQ system 80 cd/m². The maximum is
        R=G=B=125.0 corresponding to 10k cd/m².

        Windows-scRGB is displayed by Windows 10 by converting it to
        BT.2100/PQ, maintaining the CIE 1931 chromaticity and mapping the
        luminance as above. No adjustment is made to the signal to account for
        the viewing conditions.

        The reference white level of Windows-scRGB is unknown. If a reference
        white level must be assumed for compositor processing, it should be
        R=G=B=2.5375 corresponding to 203 cd/m² of Report ITU-R BT.2408-7.

        The target color volume of Windows-scRGB is unknown. The color gamut
        may be anything between sRGB and BT.2100.

        Note: EGL_EXT_gl_colorspace_scrgb_linear definition differs from
        Windows-scRGB by using R=G=B=1.0 as the reference white level, while
        Windows-scRGB reference white level is unknown or varies. However, it
        seems probable that Windows implements both
        EGL_EXT_gl_colorspace_scrgb_linear and Vulkan
        VK_COLOR_SPACE_EXTENDED_SRGB_LINEAR_EXT as Windows-scRGB.

        This request can be used when the compositor advertises
        :func:`WpColorManagerV1.feature()`.windows_scrgb. Otherwise this
        request raises the protocol error unsupported_feature.

        The resulting image description object does not allow get_information
        request. The :func:`WpImageDescriptionV1.ready()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.ready>`
        event shall be sent.

        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        """
        image_description = self._marshal_constructor(6, WpImageDescriptionV1)
        return image_description

    @WpColorManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionV1),
        Argument(ArgumentType.Object, interface=WpImageDescriptionReferenceV1),
        version=2,
    )
    def get_image_description(self, reference: WpImageDescriptionReferenceV1Proxy) -> WpImageDescriptionV1Proxy:
        """Create an image description from a reference

        This request retrieves the image description backing a reference.

        The get_information request can be used if and only if the request that
        creates the reference allows it.

        :param reference:
        :type reference:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionReferenceV1`
        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        """
        image_description = self._marshal_constructor(7, WpImageDescriptionV1, reference)
        return image_description

    @WpColorManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionV1),
        version=3,
    )
    def create_windows_bt2100(self) -> WpImageDescriptionV1Proxy:
        """Create windows-bt.2100 image description object

        This creates a pre-defined image description for the so-called Windows-
        BT.2100 stimulus encoding. This comes from the Windows 10 handling of
        its own definition of a BT.2100 color space for an HDR screen driven in
        BT.2100/PQ signalling mode.

        Windows-BT.2100 uses BT.2020 color primaries and white point. The
        transfer characteristic is st2084_pq.

        Windows-BT.2100 is generally displayed by Windows 10 without any
        adjustments to the signal to account for viewing conditions.

        The reference white level of Windows-BT.2100 is unknown. If a reference
        white level must be assumed for compositor processing, it should be 203
        cd/m² of Report ITU-R BT.2408-7.

        The target color volume of Windows-BT.2100 is unknown. The color gamut
        may be anything up to BT.2100.

        This request can be used when the compositor advertises
        :func:`WpColorManagerV1.feature()`.windows_bt2100. Otherwise this
        request raises the protocol error unsupported_feature.

        The resulting image description object does not allow get_information
        request. The :func:`WpImageDescriptionV1.ready()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.ready>`
        event shall be sent.

        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        """
        image_description = self._marshal_constructor(8, WpImageDescriptionV1)
        return image_description


class WpImageDescriptionCreatorIccV1Proxy(Proxy[WpImageDescriptionCreatorIccV1]):
    interface = WpImageDescriptionCreatorIccV1

    @WpImageDescriptionCreatorIccV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionV1),
    )
    def create(self) -> WpImageDescriptionV1Proxy:
        """Create the image description object from icc data

        Create an image description object based on the ICC information
        previously set on this object. A compositor must parse the ICC data in
        some undefined but finite amount of time.

        The completeness of the parameter set is verified. If the set is not
        complete, the protocol error incomplete_set is raised. For the
        definition of a complete set, see the description of this interface.

        If the particular combination of the information is not supported by
        the compositor, the resulting image description object shall
        immediately deliver the :func:`WpImageDescriptionV1.failed()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.failed>`
        event with the 'unsupported' cause. If a valid image description was
        created from the information, the :func:`WpImageDescriptionV1.ready()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.ready>`
        event will eventually be sent instead.

        This request destroys the :class:`WpImageDescriptionCreatorIccV1`
        object.

        The resulting image description object does not allow get_information
        request.

        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        """
        image_description = self._marshal_constructor(0, WpImageDescriptionV1)
        return image_description

    @WpImageDescriptionCreatorIccV1.request(
        Argument(ArgumentType.FileDescriptor),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def set_icc_file(self, icc_profile: int, offset: int, length: int) -> None:
        """Set the icc profile file

        Sets the ICC profile file to be used as the basis of the image
        description.

        The data shall be found through the given fd at the given offset,
        having the given length. The fd must be seekable and readable.
        Violating these requirements raises the bad_fd protocol error.

        If reading the data fails due to an error independent of the client,
        the compositor shall send the :func:`WpImageDescriptionV1.failed()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.failed>`
        event on the created
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        with the 'operating_system' cause.

        The maximum size of the ICC profile is 32 MB. If length is greater than
        that or zero, the protocol error bad_size is raised. If offset + length
        exceeds the file size, the protocol error out_of_file is raised.

        A compositor may read the file at any time starting from this request
        and only until whichever happens first: - If create request was issued,
        the
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        object   delivers either failed or ready event; or - if create request
        was not issued, this   :class:`WpImageDescriptionCreatorIccV1` object
        is destroyed.

        A compositor shall not modify the contents of the file, and the fd may
        be sealed for writes and size changes. The client must ensure to its
        best ability that the data does not change while the compositor is
        reading it.

        The data must represent a valid ICC profile. The ICC profile version
        must be 2 or 4, it must be a 3 channel profile and the class must be
        Display or ColorSpace. Violating these requirements will not result in
        a protocol error, but will eventually send the
        :func:`WpImageDescriptionV1.failed()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.failed>`
        event on the created
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        with the 'unsupported' cause.

        See the International Color Consortium specification ICC.1:2022 for
        more details about ICC profiles.

        If ICC file has already been set on this object, the protocol error
        already_set is raised.

        :param icc_profile:
            ICC profile
        :type icc_profile:
            `ArgumentType.FileDescriptor`
        :param offset:
            byte offset in fd to start of ICC data
        :type offset:
            `ArgumentType.Uint`
        :param length:
            length of ICC data in bytes
        :type length:
            `ArgumentType.Uint`
        """
        self._marshal(1, icc_profile, offset, length)


class WpImageDescriptionCreatorParamsV1Proxy(Proxy[WpImageDescriptionCreatorParamsV1]):
    interface = WpImageDescriptionCreatorParamsV1

    @WpImageDescriptionCreatorParamsV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionV1),
    )
    def create(self) -> WpImageDescriptionV1Proxy:
        """Create the image description object using params

        Create an image description object based on the parameters previously
        set on this object.

        The completeness of the parameter set is verified. If the set is not
        complete, the protocol error incomplete_set is raised. For the
        definition of a complete set, see the description of this interface.

        When both max_cll and max_fall are set, max_fall must be less or equal
        to max_cll otherwise the invalid_luminance protocol error is raised.

        In version 1, these following conditions also result in the
        invalid_luminance protocol error. Version 2 and later do not have this
        requirement. - When max_cll is set, it must be greater than min L and
        less or equal   to max L of the mastering luminance range. - When
        max_fall is set, it must be greater than min L and less or equal   to
        max L of the mastering luminance range.

        If the particular combination of the parameter set is not supported by
        the compositor, the resulting image description object shall
        immediately deliver the :func:`WpImageDescriptionV1.failed()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.failed>`
        event with the 'unsupported' cause. If a valid image description was
        created from the parameter set, the :func:`WpImageDescriptionV1.ready()
        <pywayland.protocol.color_management_v1.WpImageDescriptionV1.ready>`
        event will eventually be sent instead.

        This request destroys the :class:`WpImageDescriptionCreatorParamsV1`
        object.

        The resulting image description object does not allow get_information
        request.

        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionV1`
        """
        image_description = self._marshal_constructor(0, WpImageDescriptionV1)
        return image_description

    @WpImageDescriptionCreatorParamsV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_tf_named(self, tf: int) -> None:
        """Named transfer characteristic

        Sets the transfer characteristic using explicitly enumerated named
        functions.

        When the resulting image description is attached to an image, the
        content should be decoded according to the industry standard practices
        for the transfer characteristic.

        Only names advertised with
        :class:`~pywayland.protocol.color_management_v1.WpColorManagerV1` event
        supported_tf_named are allowed. Other values shall raise the protocol
        error invalid_tf.

        If transfer characteristic has already been set on this object, the
        protocol error already_set is raised.

        :param tf:
            named transfer function
        :type tf:
            `ArgumentType.Uint`
        """
        self._marshal(1, tf)

    @WpImageDescriptionCreatorParamsV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_tf_power(self, eexp: int) -> None:
        """Transfer characteristic as a power curve

        Sets the color component transfer characteristic to a power curve with
        the given exponent. Negative values are handled by mirroring the
        positive half of the curve through the origin. The valid domain and
        range of the curve are all finite real numbers. This curve represents
        the conversion from electrical to optical color channel values.

        The curve exponent shall be multiplied by 10000 to get the argument
        eexp value to carry the precision of 4 decimals.

        The curve exponent must be at least 1.0 and at most 10.0. Otherwise the
        protocol error invalid_tf is raised.

        If transfer characteristic has already been set on this object, the
        protocol error already_set is raised.

        This request can be used when the compositor advertises
        :func:`WpColorManagerV1.feature()
        <pywayland.protocol.color_management_v1.WpColorManagerV1.feature>`.set_tf_power.
        Otherwise this request raises the protocol error unsupported_feature.

        :param eexp:
            the exponent * 10000
        :type eexp:
            `ArgumentType.Uint`
        """
        self._marshal(2, eexp)

    @WpImageDescriptionCreatorParamsV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_primaries_named(self, primaries: int) -> None:
        """Named primaries

        Sets the color primaries and white point using explicitly named sets.
        This describes the primary color volume which is the basis for color
        value encoding.

        Only names advertised with
        :class:`~pywayland.protocol.color_management_v1.WpColorManagerV1` event
        supported_primaries_named are allowed. Other values shall raise the
        protocol error invalid_primaries_named.

        If primaries have already been set on this object, the protocol error
        already_set is raised.

        :param primaries:
            named primaries
        :type primaries:
            `ArgumentType.Uint`
        """
        self._marshal(3, primaries)

    @WpImageDescriptionCreatorParamsV1.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_primaries(self, r_x: int, r_y: int, g_x: int, g_y: int, b_x: int, b_y: int, w_x: int, w_y: int) -> None:
        """Primaries as chromaticity coordinates

        Sets the color primaries and white point using CIE 1931 xy chromaticity
        coordinates. This describes the primary color volume which is the basis
        for color value encoding.

        Each coordinate value is multiplied by 1 million to get the argument
        value to carry precision of 6 decimals.

        If primaries have already been set on this object, the protocol error
        already_set is raised.

        This request can be used if the compositor advertises
        :func:`WpColorManagerV1.feature()
        <pywayland.protocol.color_management_v1.WpColorManagerV1.feature>`.set_primaries.
        Otherwise this request raises the protocol error unsupported_feature.

        :param r_x:
            Red x * 1M
        :type r_x:
            `ArgumentType.Int`
        :param r_y:
            Red y * 1M
        :type r_y:
            `ArgumentType.Int`
        :param g_x:
            Green x * 1M
        :type g_x:
            `ArgumentType.Int`
        :param g_y:
            Green y * 1M
        :type g_y:
            `ArgumentType.Int`
        :param b_x:
            Blue x * 1M
        :type b_x:
            `ArgumentType.Int`
        :param b_y:
            Blue y * 1M
        :type b_y:
            `ArgumentType.Int`
        :param w_x:
            White x * 1M
        :type w_x:
            `ArgumentType.Int`
        :param w_y:
            White y * 1M
        :type w_y:
            `ArgumentType.Int`
        """
        self._marshal(4, r_x, r_y, g_x, g_y, b_x, b_y, w_x, w_y)

    @WpImageDescriptionCreatorParamsV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def set_luminances(self, min_lum: int, max_lum: int, reference_lum: int) -> None:
        """Primary color volume luminance range and reference white

        Sets the primary color volume luminance range and the reference white
        luminance level. These values include the minimum display emission, but
        not external flare. The minimum display emission is assumed to have the
        chromaticity of the primary color volume white point.

        The default luminances from
        https://www.color.org/chardata/rgb/srgb.xalter are - primary color
        volume minimum: 0.2 cd/m² - primary color volume maximum: 80 cd/m² -
        reference white: 80 cd/m²

        Setting a named transfer characteristic can imply other default
        luminances.

        The default luminances get overwritten when this request is used. With
        transfer_function.st2084_pq the given 'max_lum' value is ignored, and
        'max_lum' is taken as 'min_lum' + 10000 cd/m².

        'min_lum' and 'max_lum' specify the minimum and maximum luminances of
        the primary color volume as reproduced by the targeted display.

        'reference_lum' specifies the luminance of the reference white as
        reproduced by the targeted display, and reflects the targeted viewing
        environment.

        Compositors should make sure that all content is anchored, meaning that
        an input signal level of 'reference_lum' on one image description and
        another input signal level of 'reference_lum' on another image
        description should produce the same output level, even though the
        'reference_lum' on both image representations can be different.

        'reference_lum' may be higher than 'max_lum'. In that case reaching the
        reference white output level in image content requires the
        'extended_target_volume' feature support.

        If 'max_lum' or 'reference_lum' are less than or equal to 'min_lum',
        the protocol error invalid_luminance is raised.

        The minimum luminance is multiplied by 10000 to get the argument
        'min_lum' value and carries precision of 4 decimals. The maximum
        luminance and reference white luminance values are unscaled.

        If the primary color volume luminance range and the reference white
        luminance level have already been set on this object, the protocol
        error already_set is raised.

        This request can be used if the compositor advertises
        :func:`WpColorManagerV1.feature()
        <pywayland.protocol.color_management_v1.WpColorManagerV1.feature>`.set_luminances.
        Otherwise this request raises the protocol error unsupported_feature.

        :param min_lum:
            minimum luminance (cd/m²) * 10000
        :type min_lum:
            `ArgumentType.Uint`
        :param max_lum:
            maximum luminance (cd/m²)
        :type max_lum:
            `ArgumentType.Uint`
        :param reference_lum:
            reference white luminance (cd/m²)
        :type reference_lum:
            `ArgumentType.Uint`
        """
        self._marshal(5, min_lum, max_lum, reference_lum)

    @WpImageDescriptionCreatorParamsV1.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_mastering_display_primaries(self, r_x: int, r_y: int, g_x: int, g_y: int, b_x: int, b_y: int, w_x: int, w_y: int) -> None:
        """Mastering display primaries as chromaticity coordinates

        Provides the color primaries and white point of the mastering display
        using CIE 1931 xy chromaticity coordinates. This is compatible with the
        SMPTE ST 2086 definition of HDR static metadata.

        The mastering display primaries and mastering display luminances define
        the target color volume.

        If mastering display primaries are not explicitly set, the target color
        volume is assumed to have the same primaries as the primary color
        volume.

        The target color volume is defined by all tristimulus values between
        0.0 and 1.0 (inclusive) of the color space defined by the given
        mastering display primaries and white point. The colorimetry is
        identical between the container color space and the mastering display
        color space, including that no chromatic adaptation is applied even if
        the white points differ.

        The target color volume can exceed the primary color volume to allow
        for a greater color volume with an existing color space definition (for
        example scRGB). It can be smaller than the primary color volume to
        minimize gamut and tone mapping distances for big color spaces (HDR
        metadata).

        To make use of the entire target color volume a suitable pixel format
        has to be chosen (e.g. floating point to exceed the primary color
        volume, or abusing limited quantization range as with xvYCC).

        Each coordinate value is multiplied by 1 million to get the argument
        value to carry precision of 6 decimals.

        If mastering display primaries have already been set on this object,
        the protocol error already_set is raised.

        This request can be used if the compositor advertises
        :func:`WpColorManagerV1.feature()
        <pywayland.protocol.color_management_v1.WpColorManagerV1.feature>`.set_mastering_display_primaries.
        Otherwise this request raises the protocol error unsupported_feature.
        The advertisement implies support only for target color volumes fully
        contained within the primary color volume.

        If a compositor additionally supports target color volume exceeding the
        primary color volume, it must advertise
        :func:`WpColorManagerV1.feature()
        <pywayland.protocol.color_management_v1.WpColorManagerV1.feature>`.extended_target_volume.
        If a client uses target color volume exceeding the primary color volume
        and the compositor does not support it, the result is implementation
        defined. Compositors are recommended to detect this case and fail the
        image description gracefully, but it may as well result in color
        artifacts.

        :param r_x:
            Red x * 1M
        :type r_x:
            `ArgumentType.Int`
        :param r_y:
            Red y * 1M
        :type r_y:
            `ArgumentType.Int`
        :param g_x:
            Green x * 1M
        :type g_x:
            `ArgumentType.Int`
        :param g_y:
            Green y * 1M
        :type g_y:
            `ArgumentType.Int`
        :param b_x:
            Blue x * 1M
        :type b_x:
            `ArgumentType.Int`
        :param b_y:
            Blue y * 1M
        :type b_y:
            `ArgumentType.Int`
        :param w_x:
            White x * 1M
        :type w_x:
            `ArgumentType.Int`
        :param w_y:
            White y * 1M
        :type w_y:
            `ArgumentType.Int`
        """
        self._marshal(6, r_x, r_y, g_x, g_y, b_x, b_y, w_x, w_y)

    @WpImageDescriptionCreatorParamsV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def set_mastering_luminance(self, min_lum: int, max_lum: int) -> None:
        """Display mastering luminance range

        Sets the luminance range that was used during the content mastering
        process as the minimum and maximum absolute luminance L. These values
        include the minimum display emission and ambient flare luminances,
        assumed to be optically additive and have the chromaticity of the
        primary color volume white point. This should be compatible with the
        SMPTE ST 2086 definition of HDR static metadata.

        The mastering display primaries and mastering display luminances define
        the target color volume.

        If mastering luminances are not explicitly set, the target color volume
        is assumed to have the same min and max luminances as the primary color
        volume.

        If max L is less than or equal to min L, the protocol error
        invalid_luminance is raised.

        Min L value is multiplied by 10000 to get the argument min_lum value
        and carry precision of 4 decimals. Max L value is unscaled for max_lum.

        This request can be used if the compositor advertises
        :func:`WpColorManagerV1.feature()
        <pywayland.protocol.color_management_v1.WpColorManagerV1.feature>`.set_mastering_display_primaries.
        Otherwise this request raises the protocol error unsupported_feature.
        The advertisement implies support only for target color volumes fully
        contained within the primary color volume.

        If a compositor additionally supports target color volume exceeding the
        primary color volume, it must advertise
        :func:`WpColorManagerV1.feature()
        <pywayland.protocol.color_management_v1.WpColorManagerV1.feature>`.extended_target_volume.
        If a client uses target color volume exceeding the primary color volume
        and the compositor does not support it, the result is implementation
        defined. Compositors are recommended to detect this case and fail the
        image description gracefully, but it may as well result in color
        artifacts.

        :param min_lum:
            min L (cd/m²) * 10000
        :type min_lum:
            `ArgumentType.Uint`
        :param max_lum:
            max L (cd/m²)
        :type max_lum:
            `ArgumentType.Uint`
        """
        self._marshal(7, min_lum, max_lum)

    @WpImageDescriptionCreatorParamsV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_max_cll(self, max_cll: int) -> None:
        """Maximum content light level

        Sets the maximum content light level (max_cll) as defined by CTA-861-H.

        max_cll is undefined by default.

        :param max_cll:
            Maximum content light level (cd/m²)
        :type max_cll:
            `ArgumentType.Uint`
        """
        self._marshal(8, max_cll)

    @WpImageDescriptionCreatorParamsV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_max_fall(self, max_fall: int) -> None:
        """Maximum frame-average light level

        Sets the maximum frame-average light level (max_fall) as defined by
        CTA-861-H.

        max_fall is undefined by default.

        :param max_fall:
            Maximum frame-average light level (cd/m²)
        :type max_fall:
            `ArgumentType.Uint`
        """
        self._marshal(9, max_fall)


class WpImageDescriptionInfoV1Proxy(Proxy[WpImageDescriptionInfoV1]):
    interface = WpImageDescriptionInfoV1


class WpImageDescriptionReferenceV1Proxy(Proxy[WpImageDescriptionReferenceV1]):
    interface = WpImageDescriptionReferenceV1

    @WpImageDescriptionReferenceV1.request()
    def destroy(self) -> None:
        """Destroy the reference

        Destroy this object. This has no effect on the referenced image
        description.
        """
        self._marshal(0)
        self._destroy()


class WpImageDescriptionV1Proxy(Proxy[WpImageDescriptionV1]):
    interface = WpImageDescriptionV1

    @WpImageDescriptionV1.request()
    def destroy(self) -> None:
        """Destroy the image description

        Destroy this object. It is safe to destroy an object which is not
        ready.

        Destroying a :class:`WpImageDescriptionV1` object has no side-effects,
        not even if a :func:`WpColorManagementSurfaceV1.set_image_description()
        <pywayland.protocol.color_management_v1.WpColorManagementSurfaceV1.set_image_description>`
        has not yet been followed by a :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.
        """
        self._marshal(0)
        self._destroy()

    @WpImageDescriptionV1.request(
        Argument(ArgumentType.NewId, interface=WpImageDescriptionInfoV1),
    )
    def get_information(self) -> WpImageDescriptionInfoV1Proxy:
        """Get information about the image description

        Creates a
        :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionInfoV1`
        object which delivers the information that makes up the image
        description.

        Not all image description protocol objects allow get_information
        request. Whether it is allowed or not is defined by the request that
        created the object. If get_information is not allowed, the protocol
        error no_information is raised.

        :returns:
            :class:`~pywayland.protocol.color_management_v1.WpImageDescriptionInfoV1`
        """
        information = self._marshal_constructor(1, WpImageDescriptionInfoV1)
        return information


class WpColorManagementOutputV1Global(Global[WpColorManagementOutputV1]):
    interface = WpColorManagementOutputV1


class WpColorManagementSurfaceFeedbackV1Global(Global[WpColorManagementSurfaceFeedbackV1]):
    interface = WpColorManagementSurfaceFeedbackV1


class WpColorManagementSurfaceV1Global(Global[WpColorManagementSurfaceV1]):
    interface = WpColorManagementSurfaceV1


class WpColorManagerV1Global(Global[WpColorManagerV1]):
    interface = WpColorManagerV1


class WpImageDescriptionCreatorIccV1Global(Global[WpImageDescriptionCreatorIccV1]):
    interface = WpImageDescriptionCreatorIccV1


class WpImageDescriptionCreatorParamsV1Global(Global[WpImageDescriptionCreatorParamsV1]):
    interface = WpImageDescriptionCreatorParamsV1


class WpImageDescriptionInfoV1Global(Global[WpImageDescriptionInfoV1]):
    interface = WpImageDescriptionInfoV1


class WpImageDescriptionReferenceV1Global(Global[WpImageDescriptionReferenceV1]):
    interface = WpImageDescriptionReferenceV1


class WpImageDescriptionV1Global(Global[WpImageDescriptionV1]):
    interface = WpImageDescriptionV1


WpColorManagementOutputV1._gen_c()
WpColorManagementOutputV1.proxy_class = WpColorManagementOutputV1Proxy
WpColorManagementOutputV1.resource_class = WpColorManagementOutputV1Resource
WpColorManagementOutputV1.global_class = WpColorManagementOutputV1Global


WpColorManagementSurfaceFeedbackV1._gen_c()
WpColorManagementSurfaceFeedbackV1.proxy_class = WpColorManagementSurfaceFeedbackV1Proxy
WpColorManagementSurfaceFeedbackV1.resource_class = WpColorManagementSurfaceFeedbackV1Resource
WpColorManagementSurfaceFeedbackV1.global_class = WpColorManagementSurfaceFeedbackV1Global


WpColorManagementSurfaceV1._gen_c()
WpColorManagementSurfaceV1.proxy_class = WpColorManagementSurfaceV1Proxy
WpColorManagementSurfaceV1.resource_class = WpColorManagementSurfaceV1Resource
WpColorManagementSurfaceV1.global_class = WpColorManagementSurfaceV1Global


WpColorManagerV1._gen_c()
WpColorManagerV1.proxy_class = WpColorManagerV1Proxy
WpColorManagerV1.resource_class = WpColorManagerV1Resource
WpColorManagerV1.global_class = WpColorManagerV1Global


WpImageDescriptionCreatorIccV1._gen_c()
WpImageDescriptionCreatorIccV1.proxy_class = WpImageDescriptionCreatorIccV1Proxy
WpImageDescriptionCreatorIccV1.resource_class = WpImageDescriptionCreatorIccV1Resource
WpImageDescriptionCreatorIccV1.global_class = WpImageDescriptionCreatorIccV1Global


WpImageDescriptionCreatorParamsV1._gen_c()
WpImageDescriptionCreatorParamsV1.proxy_class = WpImageDescriptionCreatorParamsV1Proxy
WpImageDescriptionCreatorParamsV1.resource_class = WpImageDescriptionCreatorParamsV1Resource
WpImageDescriptionCreatorParamsV1.global_class = WpImageDescriptionCreatorParamsV1Global


WpImageDescriptionInfoV1._gen_c()
WpImageDescriptionInfoV1.proxy_class = WpImageDescriptionInfoV1Proxy
WpImageDescriptionInfoV1.resource_class = WpImageDescriptionInfoV1Resource
WpImageDescriptionInfoV1.global_class = WpImageDescriptionInfoV1Global


WpImageDescriptionReferenceV1._gen_c()
WpImageDescriptionReferenceV1.proxy_class = WpImageDescriptionReferenceV1Proxy
WpImageDescriptionReferenceV1.resource_class = WpImageDescriptionReferenceV1Resource
WpImageDescriptionReferenceV1.global_class = WpImageDescriptionReferenceV1Global


WpImageDescriptionV1._gen_c()
WpImageDescriptionV1.proxy_class = WpImageDescriptionV1Proxy
WpImageDescriptionV1.resource_class = WpImageDescriptionV1Resource
WpImageDescriptionV1.global_class = WpImageDescriptionV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2021 Simon Ser
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WpSecurityContextManagerV1(Interface):
    """Client security context manager

    This interface allows a client to register a new Wayland connection to the
    compositor and attach a security context to it.

    This is intended to be used by sandboxes. Sandbox engines attach a security
    context to all connections coming from inside the sandbox. The compositor
    can then restrict the features that the sandboxed connections can use.

    Compositors should forbid nesting multiple security contexts by not
    exposing :class:`WpSecurityContextManagerV1` global to clients with a
    security context attached, or by sending the nested protocol error. Nested
    security contexts are dangerous because they can potentially allow
    privilege escalation of a sandboxed client.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_security_context_manager_v1"
    version = 1

    class error(enum.IntEnum):
        invalid_listen_fd = 1
        nested = 2


class WpSecurityContextV1(Interface):
    """Client security context

    The security context allows a client to register a new client and attach
    security context metadata to the connections.

    When both are set, the combination of the application ID and the sandbox
    engine must uniquely identify an application. The same application ID will
    be used across instances (e.g. if the application is restarted, or if the
    application is started multiple times).

    When both are set, the combination of the instance ID and the sandbox
    engine must uniquely identify a running instance of an application.
    """

    name = "wp_security_context_v1"
    version = 1

    class error(enum.IntEnum):
        already_used = 1
        already_set = 2
        invalid_metadata = 3


class WpSecurityContextManagerV1Resource(Resource[WpSecurityContextManagerV1]):
    interface = WpSecurityContextManagerV1


class WpSecurityContextV1Resource(Resource[WpSecurityContextV1]):
    interface = WpSecurityContextV1


class WpSecurityContextManagerV1Proxy(Proxy[WpSecurityContextManagerV1]):
    interface = WpSecurityContextManagerV1

    @WpSecurityContextManagerV1.request()
    def destroy(self) -> None:
        """Destroy the manager object

        Destroy the manager. This doesn't destroy objects created with the
        manager.
        """
        self._marshal(0)
        self._destroy()

    @WpSecurityContextManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpSecurityContextV1),
        Argument(ArgumentType.FileDescriptor),
        Argument(ArgumentType.FileDescriptor),
    )
    def create_listener(self, listen_fd: int, close_fd: int) -> WpSecurityContextV1Proxy:
        """Create a new security context

        Creates a new security context with a socket listening FD.

        The compositor will accept new client connections on listen_fd.
        listen_fd must be ready to accept new connections when this request is
        sent by the client. In other words, the client must call bind(2) and
        listen(2) before sending the FD.

        close_fd is a FD that will signal hangup when the compositor should
        stop accepting new connections on listen_fd.

        The compositor must continue to accept connections on listen_fd when
        the Wayland client which created the security context disconnects.

        After sending this request, closing listen_fd and close_fd remains the
        only valid operation on them.

        :param listen_fd:
            listening socket FD
        :type listen_fd:
            `ArgumentType.FileDescriptor`
        :param close_fd:
            FD signaling when done
        :type close_fd:
            `ArgumentType.FileDescriptor`
        :returns:
            :class:`~pywayland.protocol.security_context_v1.WpSecurityContextV1`
        """
        id = self._marshal_constructor(1, WpSecurityContextV1, listen_fd, close_fd)
        return id


class WpSecurityContextV1Proxy(Proxy[WpSecurityContextV1]):
    interface = WpSecurityContextV1

    @WpSecurityContextV1.request()
    def destroy(self) -> None:
        """Destroy the security context object

        Destroy the security context object.
        """
        self._marshal(0)
        self._destroy()

    @WpSecurityContextV1.request(
        Argument(ArgumentType.String),
    )
    def set_sandbox_engine(self, name: str) -> None:
        """Set the sandbox engine

        Attach a unique sandbox engine name to the security context. The name
        should follow the reverse-DNS style (e.g. "org.flatpak").

        A list of well-known engines is maintained at:
        https://gitlab.freedesktop.org/wayland/wayland-
        protocols/-/blob/main/staging/security-context/engines.md

        It is a protocol error to call this request twice. The already_set
        error is sent in this case.

        :param name:
            the sandbox engine name
        :type name:
            `ArgumentType.String`
        """
        self._marshal(1, name)

    @WpSecurityContextV1.request(
        Argument(ArgumentType.String),
    )
    def set_app_id(self, app_id: str) -> None:
        """Set the application id

        Attach an application ID to the security context.

        The application ID is an opaque, sandbox-specific identifier for an
        application. See the well-known engines document for more details:
        https://gitlab.freedesktop.org/wayland/wayland-
        protocols/-/blob/main/staging/security-context/engines.md

        The compositor may use the application ID to group clients belonging to
        the same security context application.

        Whether this request is optional or not depends on the sandbox engine
        used.

        It is a protocol error to call this request twice. The already_set
        error is sent in this case.

        :param app_id:
            the application ID
        :type app_id:
            `ArgumentType.String`
        """
        self._marshal(2, app_id)

    @WpSecurityContextV1.request(
        Argument(ArgumentType.String),
    )
    def set_instance_id(self, instance_id: str) -> None:
        """Set the instance id

        Attach an instance ID to the security context.

        The instance ID is an opaque, sandbox-specific identifier for a running
        instance of an application. See the well-known engines document for
        more details: https://gitlab.freedesktop.org/wayland/wayland-
        protocols/-/blob/main/staging/security-context/engines.md

        Whether this request is optional or not depends on the sandbox engine
        used.

        It is a protocol error to call this request twice. The already_set
        error is sent in this case.

        :param instance_id:
            the instance ID
        :type instance_id:
            `ArgumentType.String`
        """
        self._marshal(3, instance_id)

    @WpSecurityContextV1.request()
    def commit(self) -> None:
        """Register the security context

        Atomically register the new client and attach the security context
        metadata.

        If the provided metadata is inconsistent or does not match with out of
        band metadata (see https://gitlab.freedesktop.org/wayland/wayland-
        protocols/-/blob/main/staging/security-context/engines.md), the
        invalid_metadata error may be sent eventually.

        It's a protocol error to send any request other than "destroy" after
        this request. In this case, the already_used error is sent.
        """
        self._marshal(4)


class WpSecurityContextManagerV1Global(Global[WpSecurityContextManagerV1]):
    interface = WpSecurityContextManagerV1


class WpSecurityContextV1Global(Global[WpSecurityContextV1]):
    interface = WpSecurityContextV1


WpSecurityContextManagerV1._gen_c()
WpSecurityContextManagerV1.proxy_class = WpSecurityContextManagerV1Proxy
WpSecurityContextManagerV1.resource_class = WpSecurityContextManagerV1Resource
WpSecurityContextManagerV1.global_class = WpSecurityContextManagerV1Global


WpSecurityContextV1._gen_c()
WpSecurityContextV1.proxy_class = WpSecurityContextV1Proxy
WpSecurityContextV1.resource_class = WpSecurityContextV1Resource
WpSecurityContextV1.global_class = WpSecurityContextV1Global
